# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: steammessages_clientserver_2.proto
# plugin: python-betterproto

from dataclasses import dataclass
from typing import List

import betterproto

from .steammessages_base import CMsgIPAddress


@dataclass
class CMsgClientUCMAddScreenshot(betterproto.Message):
    appid: int = betterproto.uint32_field(1)
    filename: str = betterproto.string_field(2)
    thumbname: str = betterproto.string_field(3)
    vr_filename: str = betterproto.string_field(14)
    rtime32_created: float = betterproto.fixed32_field(4)
    width: int = betterproto.uint32_field(5)
    height: int = betterproto.uint32_field(6)
    permissions: int = betterproto.uint32_field(7)
    caption: str = betterproto.string_field(8)
    shortcut_name: str = betterproto.string_field(9)
    tag: List["CMsgClientUCMAddScreenshotTag"] = betterproto.message_field(10)
    tagged_steamid: List[float] = betterproto.fixed64_field(11)
    spoiler_tag: bool = betterproto.bool_field(12)
    tagged_publishedfileid: List[int] = betterproto.uint64_field(13)


@dataclass
class CMsgClientUCMAddScreenshotTag(betterproto.Message):
    tag_name: str = betterproto.string_field(1)
    tag_value: str = betterproto.string_field(2)


@dataclass
class CMsgClientUCMAddScreenshotResponse(betterproto.Message):
    eresult: int = betterproto.int32_field(1)
    screenshotid: float = betterproto.fixed64_field(2)


@dataclass
class CMsgClientUCMDeleteScreenshot(betterproto.Message):
    screenshotid: float = betterproto.fixed64_field(1)


@dataclass
class CMsgClientUCMDeleteScreenshotResponse(betterproto.Message):
    eresult: int = betterproto.int32_field(1)


@dataclass
class CMsgClientUCMPublishFile(betterproto.Message):
    app_id: int = betterproto.uint32_field(1)
    file_name: str = betterproto.string_field(2)
    preview_file_name: str = betterproto.string_field(3)
    consumer_app_id: int = betterproto.uint32_field(4)
    title: str = betterproto.string_field(5)
    description: str = betterproto.string_field(6)
    tags: List[str] = betterproto.string_field(8)
    workshop_file: bool = betterproto.bool_field(9)
    visibility: int = betterproto.int32_field(10)
    file_type: int = betterproto.uint32_field(11)
    url: str = betterproto.string_field(12)
    video_provider: int = betterproto.uint32_field(13)
    video_account_name: str = betterproto.string_field(14)
    video_identifier: str = betterproto.string_field(15)
    in_progress: bool = betterproto.bool_field(16)


@dataclass
class CMsgClientUCMPublishFileResponse(betterproto.Message):
    eresult: int = betterproto.int32_field(1)
    published_file_id: float = betterproto.fixed64_field(2)
    needs_workshop_legal_agreement_acceptance: bool = betterproto.bool_field(3)


@dataclass
class CMsgClientUCMUpdatePublishedFile(betterproto.Message):
    app_id: int = betterproto.uint32_field(1)
    published_file_id: float = betterproto.fixed64_field(2)
    file_name: str = betterproto.string_field(3)
    preview_file_name: str = betterproto.string_field(4)
    title: str = betterproto.string_field(5)
    description: str = betterproto.string_field(6)
    tags: List[str] = betterproto.string_field(7)
    visibility: int = betterproto.int32_field(8)
    update_file: bool = betterproto.bool_field(9)
    update_preview_file: bool = betterproto.bool_field(10)
    update_title: bool = betterproto.bool_field(11)
    update_description: bool = betterproto.bool_field(12)
    update_tags: bool = betterproto.bool_field(13)
    update_visibility: bool = betterproto.bool_field(14)
    change_description: str = betterproto.string_field(15)
    update_url: bool = betterproto.bool_field(16)
    url: str = betterproto.string_field(17)
    update_content_manifest: bool = betterproto.bool_field(18)
    content_manifest: float = betterproto.fixed64_field(19)
    metadata: str = betterproto.string_field(20)
    update_metadata: bool = betterproto.bool_field(21)
    language: int = betterproto.int32_field(22)
    removed_kvtags: List[str] = betterproto.string_field(23)
    kvtags: List["CMsgClientUCMUpdatePublishedFileKeyValueTag"] = betterproto.message_field(24)
    previews: List["CMsgClientUCMUpdatePublishedFileAdditionalPreview"] = betterproto.message_field(25)
    previews_to_remove: List[int] = betterproto.int32_field(26)
    clear_in_progress: bool = betterproto.bool_field(27)
    remove_all_kvtags: bool = betterproto.bool_field(28)


@dataclass
class CMsgClientUCMUpdatePublishedFileKeyValueTag(betterproto.Message):
    key: str = betterproto.string_field(1)
    value: str = betterproto.string_field(2)


@dataclass
class CMsgClientUCMUpdatePublishedFileAdditionalPreview(betterproto.Message):
    original_file_name: str = betterproto.string_field(1)
    internal_file_name: str = betterproto.string_field(2)
    videoid: str = betterproto.string_field(3)
    preview_type: int = betterproto.uint32_field(4)
    update_index: int = betterproto.int32_field(5)


@dataclass
class CMsgClientUCMUpdatePublishedFileResponse(betterproto.Message):
    eresult: int = betterproto.int32_field(1)
    needs_workshop_legal_agreement_acceptance: bool = betterproto.bool_field(2)


@dataclass
class CMsgClientUCMDeletePublishedFile(betterproto.Message):
    published_file_id: float = betterproto.fixed64_field(1)
    app_id: int = betterproto.uint32_field(2)


@dataclass
class CMsgClientUCMDeletePublishedFileResponse(betterproto.Message):
    eresult: int = betterproto.int32_field(1)


@dataclass
class CMsgClientUCMEnumerateUserPublishedFiles(betterproto.Message):
    app_id: int = betterproto.uint32_field(1)
    start_index: int = betterproto.uint32_field(2)
    sort_order: int = betterproto.uint32_field(3)


@dataclass
class CMsgClientUCMEnumerateUserPublishedFilesResponse(betterproto.Message):
    eresult: int = betterproto.int32_field(1)
    published_files: List[
        "CMsgClientUCMEnumerateUserPublishedFilesResponsePublishedFileId"
    ] = betterproto.message_field(2)
    total_results: int = betterproto.uint32_field(3)


@dataclass
class CMsgClientUCMEnumerateUserPublishedFilesResponsePublishedFileId(betterproto.Message):
    published_file_id: float = betterproto.fixed64_field(1)


@dataclass
class CMsgClientUCMEnumerateUserSubscribedFiles(betterproto.Message):
    app_id: int = betterproto.uint32_field(1)
    start_index: int = betterproto.uint32_field(2)
    list_type: int = betterproto.uint32_field(3)
    matching_file_type: int = betterproto.uint32_field(4)
    count: int = betterproto.uint32_field(5)


@dataclass
class CMsgClientUCMEnumerateUserSubscribedFilesResponse(betterproto.Message):
    eresult: int = betterproto.int32_field(1)
    subscribed_files: List[
        "CMsgClientUCMEnumerateUserSubscribedFilesResponsePublishedFileId"
    ] = betterproto.message_field(2)
    total_results: int = betterproto.uint32_field(3)


@dataclass
class CMsgClientUCMEnumerateUserSubscribedFilesResponsePublishedFileId(betterproto.Message):
    published_file_id: float = betterproto.fixed64_field(1)
    rtime32_subscribed: float = betterproto.fixed32_field(2)


@dataclass
class CMsgClientUCMEnumerateUserSubscribedFilesWithUpdates(betterproto.Message):
    app_id: int = betterproto.uint32_field(1)
    start_index: int = betterproto.uint32_field(2)
    start_time: float = betterproto.fixed32_field(3)
    desired_revision: int = betterproto.uint32_field(4)


@dataclass
class CMsgClientUCMEnumerateUserSubscribedFilesWithUpdatesResponse(betterproto.Message):
    eresult: int = betterproto.int32_field(1)
    subscribed_files: List[
        "CMsgClientUCMEnumerateUserSubscribedFilesWithUpdatesResponsePublishedFileId"
    ] = betterproto.message_field(2)
    total_results: int = betterproto.uint32_field(3)


@dataclass
class CMsgClientUCMEnumerateUserSubscribedFilesWithUpdatesResponsePublishedFileId(betterproto.Message):
    published_file_id: float = betterproto.fixed64_field(1)
    rtime32_subscribed: float = betterproto.fixed32_field(2)
    appid: int = betterproto.uint32_field(3)
    file_hcontent: float = betterproto.fixed64_field(4)
    file_size: int = betterproto.uint32_field(5)
    rtime32_last_updated: float = betterproto.fixed32_field(6)
    is_depot_content: bool = betterproto.bool_field(7)


@dataclass
class CMsgClientUCMPublishedFileUpdated(betterproto.Message):
    published_file_id: float = betterproto.fixed64_field(1)
    app_id: int = betterproto.uint32_field(2)
    time_updated: int = betterproto.uint32_field(3)
    hcontent: float = betterproto.fixed64_field(4)
    file_size: float = betterproto.fixed32_field(5)
    is_depot_content: bool = betterproto.bool_field(6)
    revision: int = betterproto.uint32_field(7)


@dataclass
class CMsgClientWorkshopItemChangesRequest(betterproto.Message):
    app_id: int = betterproto.uint32_field(1)
    last_time_updated: int = betterproto.uint32_field(2)
    num_items_needed: int = betterproto.uint32_field(3)


@dataclass
class CMsgClientWorkshopItemChangesResponse(betterproto.Message):
    eresult: int = betterproto.int32_field(1)
    update_time: int = betterproto.uint32_field(2)
    workshop_items: List["CMsgClientWorkshopItemChangesResponseWorkshopItemInfo"] = betterproto.message_field(5)


@dataclass
class CMsgClientWorkshopItemChangesResponseWorkshopItemInfo(betterproto.Message):
    published_file_id: float = betterproto.fixed64_field(1)
    time_updated: int = betterproto.uint32_field(2)
    manifest_id: float = betterproto.fixed64_field(3)


@dataclass
class CMsgClientWorkshopItemInfoRequest(betterproto.Message):
    app_id: int = betterproto.uint32_field(1)
    last_time_updated: int = betterproto.uint32_field(2)
    workshop_items: List["CMsgClientWorkshopItemInfoRequestWorkshopItem"] = betterproto.message_field(3)


@dataclass
class CMsgClientWorkshopItemInfoRequestWorkshopItem(betterproto.Message):
    published_file_id: float = betterproto.fixed64_field(1)
    time_updated: int = betterproto.uint32_field(2)


@dataclass
class CMsgClientWorkshopItemInfoResponse(betterproto.Message):
    eresult: int = betterproto.int32_field(1)
    update_time: int = betterproto.uint32_field(2)
    workshop_items: List["CMsgClientWorkshopItemInfoResponseWorkshopItemInfo"] = betterproto.message_field(3)
    private_items: List[float] = betterproto.fixed64_field(4)


@dataclass
class CMsgClientWorkshopItemInfoResponseWorkshopItemInfo(betterproto.Message):
    published_file_id: float = betterproto.fixed64_field(1)
    time_updated: int = betterproto.uint32_field(2)
    manifest_id: float = betterproto.fixed64_field(3)
    is_legacy: bool = betterproto.bool_field(4)


@dataclass
class CMsgClientUCMGetPublishedFilesForUser(betterproto.Message):
    app_id: int = betterproto.uint32_field(1)
    creator_steam_id: float = betterproto.fixed64_field(2)
    required_tags: List[str] = betterproto.string_field(3)
    excluded_tags: List[str] = betterproto.string_field(4)
    start_index: int = betterproto.uint32_field(5)


@dataclass
class CMsgClientUCMGetPublishedFilesForUserResponse(betterproto.Message):
    eresult: int = betterproto.int32_field(1)
    published_files: List["CMsgClientUCMGetPublishedFilesForUserResponsePublishedFileId"] = betterproto.message_field(2)
    total_results: int = betterproto.uint32_field(3)


@dataclass
class CMsgClientUCMGetPublishedFilesForUserResponsePublishedFileId(betterproto.Message):
    published_file_id: float = betterproto.fixed64_field(1)


@dataclass
class CMsgClientUCMSetUserPublishedFileAction(betterproto.Message):
    published_file_id: float = betterproto.fixed64_field(1)
    app_id: int = betterproto.uint32_field(2)
    action: int = betterproto.int32_field(3)


@dataclass
class CMsgClientUCMSetUserPublishedFileActionResponse(betterproto.Message):
    eresult: int = betterproto.int32_field(1)


@dataclass
class CMsgClientUCMEnumeratePublishedFilesByUserAction(betterproto.Message):
    app_id: int = betterproto.uint32_field(1)
    start_index: int = betterproto.uint32_field(2)
    action: int = betterproto.int32_field(3)


@dataclass
class CMsgClientUCMEnumeratePublishedFilesByUserActionResponse(betterproto.Message):
    eresult: int = betterproto.int32_field(1)
    published_files: List[
        "CMsgClientUCMEnumeratePublishedFilesByUserActionResponsePublishedFileId"
    ] = betterproto.message_field(2)
    total_results: int = betterproto.uint32_field(3)


@dataclass
class CMsgClientUCMEnumeratePublishedFilesByUserActionResponsePublishedFileId(betterproto.Message):
    published_file_id: float = betterproto.fixed64_field(1)
    rtime_time_stamp: float = betterproto.fixed32_field(2)


@dataclass
class CMsgClientScreenshotsChanged(betterproto.Message):
    pass


@dataclass
class CMsgClientUpdateUserGameInfo(betterproto.Message):
    steamid_idgs: float = betterproto.fixed64_field(1)
    gameid: float = betterproto.fixed64_field(2)
    game_ip: int = betterproto.uint32_field(3)
    game_port: int = betterproto.uint32_field(4)
    token: bytes = betterproto.bytes_field(5)


@dataclass
class CMsgClientRichPresenceUpload(betterproto.Message):
    rich_presence_kv: bytes = betterproto.bytes_field(1)
    steamid_broadcast: List[float] = betterproto.fixed64_field(2)


@dataclass
class CMsgClientRichPresenceRequest(betterproto.Message):
    steamid_request: List[float] = betterproto.fixed64_field(1)


@dataclass
class CMsgClientRichPresenceInfo(betterproto.Message):
    rich_presence: List["CMsgClientRichPresenceInfoRichPresence"] = betterproto.message_field(1)


@dataclass
class CMsgClientRichPresenceInfoRichPresence(betterproto.Message):
    steamid_user: float = betterproto.fixed64_field(1)
    rich_presence_kv: bytes = betterproto.bytes_field(2)


@dataclass
class CMsgClientCheckFileSignature(betterproto.Message):
    app_id: int = betterproto.uint32_field(1)


@dataclass
class CMsgClientCheckFileSignatureResponse(betterproto.Message):
    app_id: int = betterproto.uint32_field(1)
    pid: int = betterproto.uint32_field(2)
    eresult: int = betterproto.uint32_field(3)
    filename: str = betterproto.string_field(4)
    esignatureresult: int = betterproto.uint32_field(5)
    sha_file: bytes = betterproto.bytes_field(6)
    signatureheader: bytes = betterproto.bytes_field(7)
    filesize: int = betterproto.uint32_field(8)
    getlasterror: int = betterproto.uint32_field(9)
    evalvesignaturecheckdetail: int = betterproto.uint32_field(10)


@dataclass
class CMsgClientRegisterKey(betterproto.Message):
    key: str = betterproto.string_field(1)


@dataclass
class CMsgClientPurchaseResponse(betterproto.Message):
    eresult: int = betterproto.int32_field(1)
    purchase_result_details: int = betterproto.int32_field(2)
    purchase_receipt_info: bytes = betterproto.bytes_field(3)


@dataclass
class CMsgClientActivateOEMLicense(betterproto.Message):
    bios_manufacturer: str = betterproto.string_field(1)
    bios_serialnumber: str = betterproto.string_field(2)
    license_file: bytes = betterproto.bytes_field(3)
    mainboard_manufacturer: str = betterproto.string_field(4)
    mainboard_product: str = betterproto.string_field(5)
    mainboard_serialnumber: str = betterproto.string_field(6)


@dataclass
class CMsgClientRegisterOEMMachine(betterproto.Message):
    oem_register_file: bytes = betterproto.bytes_field(1)


@dataclass
class CMsgClientRegisterOEMMachineResponse(betterproto.Message):
    eresult: int = betterproto.uint32_field(1)


@dataclass
class CMsgClientPurchaseWithMachineID(betterproto.Message):
    package_id: int = betterproto.uint32_field(1)
    machine_info: bytes = betterproto.bytes_field(2)


@dataclass
class CMsgTrading_InitiateTradeRequest(betterproto.Message):
    trade_request_id: int = betterproto.uint32_field(1)
    other_steamid: int = betterproto.uint64_field(2)
    other_name: str = betterproto.string_field(3)


@dataclass
class CMsgTrading_InitiateTradeResponse(betterproto.Message):
    response: int = betterproto.uint32_field(1)
    trade_request_id: int = betterproto.uint32_field(2)
    other_steamid: int = betterproto.uint64_field(3)
    steamguard_required_days: int = betterproto.uint32_field(4)
    new_device_cooldown_days: int = betterproto.uint32_field(5)
    default_password_reset_probation_days: int = betterproto.uint32_field(6)
    password_reset_probation_days: int = betterproto.uint32_field(7)
    default_email_change_probation_days: int = betterproto.uint32_field(8)
    email_change_probation_days: int = betterproto.uint32_field(9)


@dataclass
class CMsgTrading_CancelTradeRequest(betterproto.Message):
    other_steamid: int = betterproto.uint64_field(1)


@dataclass
class CMsgTrading_StartSession(betterproto.Message):
    other_steamid: int = betterproto.uint64_field(1)


@dataclass
class CMsgClientGetCDNAuthToken(betterproto.Message):
    depot_id: int = betterproto.uint32_field(1)
    host_name: str = betterproto.string_field(2)
    app_id: int = betterproto.uint32_field(3)


@dataclass
class CMsgClientGetDepotDecryptionKey(betterproto.Message):
    depot_id: int = betterproto.uint32_field(1)
    app_id: int = betterproto.uint32_field(2)


@dataclass
class CMsgClientGetDepotDecryptionKeyResponse(betterproto.Message):
    eresult: int = betterproto.int32_field(1)
    depot_id: int = betterproto.uint32_field(2)
    depot_encryption_key: bytes = betterproto.bytes_field(3)


@dataclass
class CMsgClientCheckAppBetaPassword(betterproto.Message):
    app_id: int = betterproto.uint32_field(1)
    betapassword: str = betterproto.string_field(2)


@dataclass
class CMsgClientCheckAppBetaPasswordResponse(betterproto.Message):
    eresult: int = betterproto.int32_field(1)
    betapasswords: List["CMsgClientCheckAppBetaPasswordResponseBetaPassword"] = betterproto.message_field(4)


@dataclass
class CMsgClientCheckAppBetaPasswordResponseBetaPassword(betterproto.Message):
    betaname: str = betterproto.string_field(1)
    betapassword: str = betterproto.string_field(2)


@dataclass
class CMsgClientUpdateAppJobReport(betterproto.Message):
    app_id: int = betterproto.uint32_field(1)
    depot_ids: List[int] = betterproto.uint32_field(2)
    app_state: int = betterproto.uint32_field(3)
    job_app_error: int = betterproto.uint32_field(4)
    error_details: str = betterproto.string_field(5)
    job_duration: int = betterproto.uint32_field(6)
    files_validation_failed: int = betterproto.uint32_field(7)
    job_bytes_downloaded: int = betterproto.uint64_field(8)
    job_bytes_staged: int = betterproto.uint64_field(9)
    bytes_comitted: int = betterproto.uint64_field(10)
    start_app_state: int = betterproto.uint32_field(11)
    stats_machine_id: float = betterproto.fixed64_field(12)
    branch_name: str = betterproto.string_field(13)
    total_bytes_downloaded: int = betterproto.uint64_field(14)
    total_bytes_staged: int = betterproto.uint64_field(15)
    total_bytes_restored: int = betterproto.uint64_field(16)
    is_borrowed: bool = betterproto.bool_field(17)
    is_free_weekend: bool = betterproto.bool_field(18)
    total_bytes_legacy: int = betterproto.uint64_field(19)
    total_bytes_patched: int = betterproto.uint64_field(20)
    total_bytes_saved: int = betterproto.uint64_field(21)
    cell_id: int = betterproto.uint32_field(22)


@dataclass
class CMsgClientDPContentStatsReport(betterproto.Message):
    stats_machine_id: float = betterproto.fixed64_field(1)
    country_code: str = betterproto.string_field(2)
    os_type: int = betterproto.int32_field(3)
    language: int = betterproto.int32_field(4)
    num_install_folders: int = betterproto.uint32_field(5)
    num_installed_games: int = betterproto.uint32_field(6)
    size_installed_games: int = betterproto.uint64_field(7)


@dataclass
class CMsgClientGetCDNAuthTokenResponse(betterproto.Message):
    eresult: int = betterproto.uint32_field(1)
    token: str = betterproto.string_field(2)
    expiration_time: int = betterproto.uint32_field(3)


@dataclass
class CMsgDownloadRateStatistics(betterproto.Message):
    cell_id: int = betterproto.uint32_field(1)
    stats: List["CMsgDownloadRateStatisticsStatsInfo"] = betterproto.message_field(2)
    throttling_kbps: int = betterproto.uint32_field(3)


@dataclass
class CMsgDownloadRateStatisticsStatsInfo(betterproto.Message):
    source_type: int = betterproto.uint32_field(1)
    source_id: int = betterproto.uint32_field(2)
    seconds: int = betterproto.uint32_field(3)
    bytes: int = betterproto.uint64_field(4)
    host_name: str = betterproto.string_field(5)
    microseconds: int = betterproto.uint64_field(6)
    used_ipv6: bool = betterproto.bool_field(7)
    proxied: bool = betterproto.bool_field(8)


@dataclass
class CMsgClientRequestAccountData(betterproto.Message):
    account_or_email: str = betterproto.string_field(1)
    action: int = betterproto.uint32_field(2)


@dataclass
class CMsgClientRequestAccountDataResponse(betterproto.Message):
    action: int = betterproto.uint32_field(1)
    eresult: int = betterproto.uint32_field(2)
    account_name: str = betterproto.string_field(3)
    ct_matches: int = betterproto.uint32_field(4)
    account_name_suggestion1: str = betterproto.string_field(5)
    account_name_suggestion2: str = betterproto.string_field(6)
    account_name_suggestion3: str = betterproto.string_field(7)


@dataclass
class CMsgClientUGSGetGlobalStats(betterproto.Message):
    gameid: int = betterproto.uint64_field(1)
    history_days_requested: int = betterproto.uint32_field(2)
    time_last_requested: float = betterproto.fixed32_field(3)
    first_day_cached: int = betterproto.uint32_field(4)
    days_cached: int = betterproto.uint32_field(5)


@dataclass
class CMsgClientUGSGetGlobalStatsResponse(betterproto.Message):
    eresult: int = betterproto.int32_field(1)
    timestamp: float = betterproto.fixed32_field(2)
    day_current: int = betterproto.int32_field(3)
    days: List["CMsgClientUGSGetGlobalStatsResponseDay"] = betterproto.message_field(4)


@dataclass
class CMsgClientUGSGetGlobalStatsResponseDay(betterproto.Message):
    day_id: int = betterproto.uint32_field(1)
    stats: List["CMsgClientUGSGetGlobalStatsResponseDayStat"] = betterproto.message_field(2)


@dataclass
class CMsgClientUGSGetGlobalStatsResponseDayStat(betterproto.Message):
    stat_id: int = betterproto.int32_field(1)
    data: int = betterproto.int64_field(2)


@dataclass
class CMsgGameServerData(betterproto.Message):
    steam_id_gs: float = betterproto.fixed64_field(1)
    deprecated_ip: int = betterproto.uint32_field(2)
    query_port: int = betterproto.uint32_field(3)
    game_port: int = betterproto.uint32_field(4)
    sourcetv_port: int = betterproto.uint32_field(5)
    name: str = betterproto.string_field(22)
    game_ip_address: "CMsgIPAddress" = betterproto.message_field(23)
    app_id: int = betterproto.uint32_field(6)
    gamedir: str = betterproto.string_field(7)
    version: str = betterproto.string_field(8)
    product: str = betterproto.string_field(9)
    region: str = betterproto.string_field(10)
    players: List["CMsgGameServerDataPlayer"] = betterproto.message_field(11)
    max_players: int = betterproto.uint32_field(12)
    bot_count: int = betterproto.uint32_field(13)
    password: bool = betterproto.bool_field(14)
    secure: bool = betterproto.bool_field(15)
    dedicated: bool = betterproto.bool_field(16)
    os: str = betterproto.string_field(17)
    game_data: str = betterproto.string_field(18)
    game_data_version: int = betterproto.uint32_field(19)
    game_type: str = betterproto.string_field(20)
    map: str = betterproto.string_field(21)


@dataclass
class CMsgGameServerDataPlayer(betterproto.Message):
    steam_id: float = betterproto.fixed64_field(1)


@dataclass
class CMsgGameServerRemove(betterproto.Message):
    steam_id: float = betterproto.fixed64_field(1)
    deprecated_ip: int = betterproto.uint32_field(2)
    query_port: int = betterproto.uint32_field(3)
    ip: "CMsgIPAddress" = betterproto.message_field(4)


@dataclass
class CMsgClientGMSServerQuery(betterproto.Message):
    app_id: int = betterproto.uint32_field(1)
    geo_location_ip: int = betterproto.uint32_field(2)
    region_code: int = betterproto.uint32_field(3)
    filter_text: str = betterproto.string_field(4)
    max_servers: int = betterproto.uint32_field(5)


@dataclass
class CMsgGMSClientServerQueryResponse(betterproto.Message):
    servers: List["CMsgGMSClientServerQueryResponseServer"] = betterproto.message_field(1)
    error: str = betterproto.string_field(2)


@dataclass
class CMsgGMSClientServerQueryResponseServer(betterproto.Message):
    deprecated_server_ip: int = betterproto.uint32_field(1)
    server_port: int = betterproto.uint32_field(2)
    auth_players: int = betterproto.uint32_field(3)
    server_ip: "CMsgIPAddress" = betterproto.message_field(4)


@dataclass
class CMsgGameServerOutOfDate(betterproto.Message):
    steam_id_gs: float = betterproto.fixed64_field(1)
    reject: bool = betterproto.bool_field(2)
    message: str = betterproto.string_field(3)


@dataclass
class CMsgClientRedeemGuestPass(betterproto.Message):
    guest_pass_id: float = betterproto.fixed64_field(1)


@dataclass
class CMsgClientRedeemGuestPassResponse(betterproto.Message):
    eresult: int = betterproto.uint32_field(1)
    package_id: int = betterproto.uint32_field(2)
    must_own_appid: int = betterproto.uint32_field(3)


@dataclass
class CMsgClientGetClanActivityCounts(betterproto.Message):
    steamid_clans: List[int] = betterproto.uint64_field(1)


@dataclass
class CMsgClientGetClanActivityCountsResponse(betterproto.Message):
    eresult: int = betterproto.uint32_field(1)


@dataclass
class CMsgClientOGSReportString(betterproto.Message):
    accumulated: bool = betterproto.bool_field(1)
    sessionid: int = betterproto.uint64_field(2)
    severity: int = betterproto.int32_field(3)
    formatter: str = betterproto.string_field(4)
    varargs: bytes = betterproto.bytes_field(5)


@dataclass
class CMsgClientOGSReportBug(betterproto.Message):
    sessionid: int = betterproto.uint64_field(1)
    bugtext: str = betterproto.string_field(2)
    screenshot: bytes = betterproto.bytes_field(3)


@dataclass
class CMsgGSAssociateWithClan(betterproto.Message):
    steam_id_clan: float = betterproto.fixed64_field(1)


@dataclass
class CMsgGSAssociateWithClanResponse(betterproto.Message):
    steam_id_clan: float = betterproto.fixed64_field(1)
    eresult: int = betterproto.uint32_field(2)


@dataclass
class CMsgGSComputeNewPlayerCompatibility(betterproto.Message):
    steam_id_candidate: float = betterproto.fixed64_field(1)


@dataclass
class CMsgGSComputeNewPlayerCompatibilityResponse(betterproto.Message):
    steam_id_candidate: float = betterproto.fixed64_field(1)
    eresult: int = betterproto.uint32_field(2)
    is_clan_member: bool = betterproto.bool_field(3)
    ct_dont_like_you: int = betterproto.int32_field(4)
    ct_you_dont_like: int = betterproto.int32_field(5)
    ct_clanmembers_dont_like_you: int = betterproto.int32_field(6)


@dataclass
class CMsgClientSentLogs(betterproto.Message):
    pass


@dataclass
class CMsgGCClient(betterproto.Message):
    appid: int = betterproto.uint32_field(1)
    msgtype: int = betterproto.uint32_field(2)
    payload: bytes = betterproto.bytes_field(3)
    steamid: float = betterproto.fixed64_field(4)
    gcname: str = betterproto.string_field(5)
    ip: int = betterproto.uint32_field(6)


@dataclass
class CMsgClientRequestFreeLicense(betterproto.Message):
    appids: List[int] = betterproto.uint32_field(2)


@dataclass
class CMsgClientRequestFreeLicenseResponse(betterproto.Message):
    eresult: int = betterproto.uint32_field(1)
    granted_packageids: List[int] = betterproto.uint32_field(2)
    granted_appids: List[int] = betterproto.uint32_field(3)


@dataclass
class CMsgDRMDownloadRequestWithCrashData(betterproto.Message):
    download_flags: int = betterproto.uint32_field(1)
    download_types_known: int = betterproto.uint32_field(2)
    guid_drm: bytes = betterproto.bytes_field(3)
    guid_split: bytes = betterproto.bytes_field(4)
    guid_merge: bytes = betterproto.bytes_field(5)
    module_name: str = betterproto.string_field(6)
    module_path: str = betterproto.string_field(7)
    crash_data: bytes = betterproto.bytes_field(8)


@dataclass
class CMsgDRMDownloadResponse(betterproto.Message):
    eresult: int = betterproto.uint32_field(1)
    app_id: int = betterproto.uint32_field(2)
    blob_download_type: int = betterproto.uint32_field(3)
    merge_guid: bytes = betterproto.bytes_field(4)
    download_file_dfs_ip: int = betterproto.uint32_field(5)
    download_file_dfs_port: int = betterproto.uint32_field(6)
    download_file_url: str = betterproto.string_field(7)
    module_path: str = betterproto.string_field(8)


@dataclass
class CMsgDRMFinalResult(betterproto.Message):
    e_result: int = betterproto.uint32_field(1)
    app_id: int = betterproto.uint32_field(2)
    blob_download_type: int = betterproto.uint32_field(3)
    error_detail: int = betterproto.uint32_field(4)
    merge_guid: bytes = betterproto.bytes_field(5)
    download_file_dfs_ip: int = betterproto.uint32_field(6)
    download_file_dfs_port: int = betterproto.uint32_field(7)
    download_file_url: str = betterproto.string_field(8)


@dataclass
class CMsgClientDPCheckSpecialSurvey(betterproto.Message):
    survey_id: int = betterproto.uint32_field(1)


@dataclass
class CMsgClientDPCheckSpecialSurveyResponse(betterproto.Message):
    e_result: int = betterproto.uint32_field(1)
    state: int = betterproto.uint32_field(2)
    name: str = betterproto.string_field(3)
    custom_url: str = betterproto.string_field(4)
    include_software: bool = betterproto.bool_field(5)
    token: bytes = betterproto.bytes_field(6)


@dataclass
class CMsgClientDPSendSpecialSurveyResponse(betterproto.Message):
    survey_id: int = betterproto.uint32_field(1)
    data: bytes = betterproto.bytes_field(2)


@dataclass
class CMsgClientDPSendSpecialSurveyResponseReply(betterproto.Message):
    e_result: int = betterproto.uint32_field(1)
    token: bytes = betterproto.bytes_field(2)


@dataclass
class CMsgClientRequestForgottenPasswordEmail(betterproto.Message):
    account_name: str = betterproto.string_field(1)
    password_tried: str = betterproto.string_field(2)


@dataclass
class CMsgClientRequestForgottenPasswordEmailResponse(betterproto.Message):
    e_result: int = betterproto.uint32_field(1)
    use_secret_question: bool = betterproto.bool_field(2)


@dataclass
class CMsgClientItemAnnouncements(betterproto.Message):
    count_new_items: int = betterproto.uint32_field(1)
    unseen_items: List["CMsgClientItemAnnouncementsUnseenItem"] = betterproto.message_field(2)


@dataclass
class CMsgClientItemAnnouncementsUnseenItem(betterproto.Message):
    appid: int = betterproto.uint32_field(1)
    context_id: int = betterproto.uint64_field(2)
    asset_id: int = betterproto.uint64_field(3)
    amount: int = betterproto.uint64_field(4)
    rtime32_gained: float = betterproto.fixed32_field(5)
    source_appid: int = betterproto.uint32_field(6)


@dataclass
class CMsgClientRequestItemAnnouncements(betterproto.Message):
    pass


@dataclass
class CMsgClientUserNotifications(betterproto.Message):
    notifications: List["CMsgClientUserNotificationsNotification"] = betterproto.message_field(1)


@dataclass
class CMsgClientUserNotificationsNotification(betterproto.Message):
    type: int = betterproto.uint32_field(1)
    count: int = betterproto.uint32_field(2)


@dataclass
class CMsgClientCommentNotifications(betterproto.Message):
    count_new_comments: int = betterproto.uint32_field(1)
    count_new_comments_owner: int = betterproto.uint32_field(2)
    count_new_comments_subscriptions: int = betterproto.uint32_field(3)


@dataclass
class CMsgClientRequestCommentNotifications(betterproto.Message):
    pass


@dataclass
class CMsgClientOfflineMessageNotification(betterproto.Message):
    offline_messages: int = betterproto.uint32_field(1)
    friends_with_offline_messages: List[int] = betterproto.uint32_field(2)


@dataclass
class CMsgClientRequestOfflineMessageCount(betterproto.Message):
    pass


@dataclass
class CMsgClientChatGetFriendMessageHistory(betterproto.Message):
    steamid: float = betterproto.fixed64_field(1)


@dataclass
class CMsgClientChatGetFriendMessageHistoryResponse(betterproto.Message):
    steamid: float = betterproto.fixed64_field(1)
    success: int = betterproto.uint32_field(2)
    messages: List["CMsgClientChatGetFriendMessageHistoryResponseFriendMessage"] = betterproto.message_field(3)


@dataclass
class CMsgClientChatGetFriendMessageHistoryResponseFriendMessage(betterproto.Message):
    accountid: int = betterproto.uint32_field(1)
    timestamp: int = betterproto.uint32_field(2)
    message: str = betterproto.string_field(3)
    unread: bool = betterproto.bool_field(4)


@dataclass
class CMsgClientChatGetFriendMessageHistoryForOfflineMessages(betterproto.Message):
    pass


@dataclass
class CMsgClientFSGetFriendsSteamLevels(betterproto.Message):
    accountids: List[int] = betterproto.uint32_field(1)


@dataclass
class CMsgClientFSGetFriendsSteamLevelsResponse(betterproto.Message):
    friends: List["CMsgClientFSGetFriendsSteamLevelsResponseFriend"] = betterproto.message_field(1)


@dataclass
class CMsgClientFSGetFriendsSteamLevelsResponseFriend(betterproto.Message):
    accountid: int = betterproto.uint32_field(1)
    level: int = betterproto.uint32_field(2)


@dataclass
class CMsgClientEmailAddrInfo(betterproto.Message):
    email_address: str = betterproto.string_field(1)
    email_is_validated: bool = betterproto.bool_field(2)
    email_validation_changed: bool = betterproto.bool_field(3)
    credential_change_requires_code: bool = betterproto.bool_field(4)
    password_or_secretqa_change_requires_code: bool = betterproto.bool_field(5)
    remind_user_about_email: bool = betterproto.bool_field(6)


@dataclass
class CMsgCREItemVoteSummary(betterproto.Message):
    published_file_ids: List["CMsgCREItemVoteSummaryPublishedFileId"] = betterproto.message_field(1)


@dataclass
class CMsgCREItemVoteSummaryPublishedFileId(betterproto.Message):
    published_file_id: float = betterproto.fixed64_field(1)


@dataclass
class CMsgCREItemVoteSummaryResponse(betterproto.Message):
    eresult: int = betterproto.int32_field(1)
    item_vote_summaries: List["CMsgCREItemVoteSummaryResponseItemVoteSummary"] = betterproto.message_field(2)


@dataclass
class CMsgCREItemVoteSummaryResponseItemVoteSummary(betterproto.Message):
    published_file_id: float = betterproto.fixed64_field(1)
    votes_for: int = betterproto.int32_field(2)
    votes_against: int = betterproto.int32_field(3)
    reports: int = betterproto.int32_field(4)
    score: float = betterproto.float_field(5)


@dataclass
class CMsgCREUpdateUserPublishedItemVote(betterproto.Message):
    published_file_id: float = betterproto.fixed64_field(1)
    vote_up: bool = betterproto.bool_field(2)


@dataclass
class CMsgCREUpdateUserPublishedItemVoteResponse(betterproto.Message):
    eresult: int = betterproto.int32_field(1)


@dataclass
class CMsgCREGetUserPublishedItemVoteDetails(betterproto.Message):
    published_file_ids: List["CMsgCREGetUserPublishedItemVoteDetailsPublishedFileId"] = betterproto.message_field(1)


@dataclass
class CMsgCREGetUserPublishedItemVoteDetailsPublishedFileId(betterproto.Message):
    published_file_id: float = betterproto.fixed64_field(1)


@dataclass
class CMsgCREGetUserPublishedItemVoteDetailsResponse(betterproto.Message):
    eresult: int = betterproto.int32_field(1)
    user_item_vote_details: List[
        "CMsgCREGetUserPublishedItemVoteDetailsResponseUserItemVoteDetail"
    ] = betterproto.message_field(2)


@dataclass
class CMsgCREGetUserPublishedItemVoteDetailsResponseUserItemVoteDetail(betterproto.Message):
    published_file_id: float = betterproto.fixed64_field(1)
    vote: int = betterproto.int32_field(2)


@dataclass
class CMsgGameServerPingSample(betterproto.Message):
    my_ip: float = betterproto.fixed32_field(1)
    gs_app_id: int = betterproto.int32_field(2)
    gs_samples: List["CMsgGameServerPingSampleSample"] = betterproto.message_field(3)


@dataclass
class CMsgGameServerPingSampleSample(betterproto.Message):
    ip: float = betterproto.fixed32_field(1)
    avg_ping_ms: int = betterproto.uint32_field(2)
    stddev_ping_ms_x10: int = betterproto.uint32_field(3)


@dataclass
class CMsgFSGetFollowerCount(betterproto.Message):
    steam_id: float = betterproto.fixed64_field(1)


@dataclass
class CMsgFSGetFollowerCountResponse(betterproto.Message):
    eresult: int = betterproto.int32_field(1)
    count: int = betterproto.int32_field(2)


@dataclass
class CMsgFSGetIsFollowing(betterproto.Message):
    steam_id: float = betterproto.fixed64_field(1)


@dataclass
class CMsgFSGetIsFollowingResponse(betterproto.Message):
    eresult: int = betterproto.int32_field(1)
    is_following: bool = betterproto.bool_field(2)


@dataclass
class CMsgFSEnumerateFollowingList(betterproto.Message):
    start_index: int = betterproto.uint32_field(1)


@dataclass
class CMsgFSEnumerateFollowingListResponse(betterproto.Message):
    eresult: int = betterproto.int32_field(1)
    total_results: int = betterproto.int32_field(2)
    steam_ids: List[float] = betterproto.fixed64_field(3)


@dataclass
class CMsgDPGetNumberOfCurrentPlayers(betterproto.Message):
    appid: int = betterproto.uint32_field(1)


@dataclass
class CMsgDPGetNumberOfCurrentPlayersResponse(betterproto.Message):
    eresult: int = betterproto.int32_field(1)
    player_count: int = betterproto.int32_field(2)


@dataclass
class CMsgClientFriendUserStatusPublished(betterproto.Message):
    friend_steamid: float = betterproto.fixed64_field(1)
    appid: int = betterproto.uint32_field(2)
    status_text: str = betterproto.string_field(3)


@dataclass
class CMsgClientServiceMethodLegacy(betterproto.Message):
    method_name: str = betterproto.string_field(1)
    serialized_method: bytes = betterproto.bytes_field(2)
    is_notification: bool = betterproto.bool_field(3)


@dataclass
class CMsgClientServiceMethodLegacyResponse(betterproto.Message):
    method_name: str = betterproto.string_field(1)
    serialized_method_response: bytes = betterproto.bytes_field(2)


@dataclass
class CMsgClientUIMode(betterproto.Message):
    uimode: int = betterproto.uint32_field(1)
    chat_mode: int = betterproto.uint32_field(2)


@dataclass
class CMsgClientVanityURLChangedNotification(betterproto.Message):
    vanity_url: str = betterproto.string_field(1)


@dataclass
class CMsgClientAuthorizeLocalDeviceRequest(betterproto.Message):
    device_description: str = betterproto.string_field(1)
    owner_account_id: int = betterproto.uint32_field(2)
    local_device_token: int = betterproto.uint64_field(3)


@dataclass
class CMsgClientAuthorizeLocalDevice(betterproto.Message):
    eresult: int = betterproto.int32_field(1)
    owner_account_id: int = betterproto.uint32_field(2)
    authed_device_token: int = betterproto.uint64_field(3)


@dataclass
class CMsgClientAuthorizeLocalDeviceNotification(betterproto.Message):
    eresult: int = betterproto.int32_field(1)
    owner_account_id: int = betterproto.uint32_field(2)
    local_device_token: int = betterproto.uint64_field(3)


@dataclass
class CMsgClientDeauthorizeDeviceRequest(betterproto.Message):
    deauthorization_account_id: int = betterproto.uint32_field(1)
    deauthorization_device_token: int = betterproto.uint64_field(2)


@dataclass
class CMsgClientDeauthorizeDevice(betterproto.Message):
    eresult: int = betterproto.int32_field(1)
    deauthorization_account_id: int = betterproto.uint32_field(2)


@dataclass
class CMsgClientUseLocalDeviceAuthorizations(betterproto.Message):
    authorization_account_id: List[int] = betterproto.uint32_field(1)
    device_tokens: List["CMsgClientUseLocalDeviceAuthorizationsDeviceToken"] = betterproto.message_field(2)


@dataclass
class CMsgClientUseLocalDeviceAuthorizationsDeviceToken(betterproto.Message):
    owner_account_id: int = betterproto.uint32_field(1)
    token_id: int = betterproto.uint64_field(2)


@dataclass
class CMsgClientGetAuthorizedDevices(betterproto.Message):
    pass


@dataclass
class CMsgClientGetAuthorizedDevicesResponse(betterproto.Message):
    eresult: int = betterproto.int32_field(1)
    authorized_device: List["CMsgClientGetAuthorizedDevicesResponseAuthorizedDevice"] = betterproto.message_field(2)


@dataclass
class CMsgClientGetAuthorizedDevicesResponseAuthorizedDevice(betterproto.Message):
    auth_device_token: int = betterproto.uint64_field(1)
    device_name: str = betterproto.string_field(2)
    last_access_time: int = betterproto.uint32_field(3)
    borrower_id: int = betterproto.uint32_field(4)
    is_pending: bool = betterproto.bool_field(5)
    app_played: int = betterproto.uint32_field(6)


@dataclass
class CMsgClientSharedLibraryLockStatus(betterproto.Message):
    locked_library: List["CMsgClientSharedLibraryLockStatusLockedLibrary"] = betterproto.message_field(1)
    own_library_locked_by: int = betterproto.uint32_field(2)


@dataclass
class CMsgClientSharedLibraryLockStatusLockedLibrary(betterproto.Message):
    owner_id: int = betterproto.uint32_field(1)
    locked_by: int = betterproto.uint32_field(2)


@dataclass
class CMsgClientSharedLibraryStopPlaying(betterproto.Message):
    seconds_left: int = betterproto.int32_field(1)
    stop_apps: List["CMsgClientSharedLibraryStopPlayingStopApp"] = betterproto.message_field(2)


@dataclass
class CMsgClientSharedLibraryStopPlayingStopApp(betterproto.Message):
    app_id: int = betterproto.uint32_field(1)
    owner_id: int = betterproto.uint32_field(2)


@dataclass
class CMsgClientServiceCall(betterproto.Message):
    sysid_routing: bytes = betterproto.bytes_field(1)
    call_handle: int = betterproto.uint32_field(2)
    module_crc: int = betterproto.uint32_field(3)
    module_hash: bytes = betterproto.bytes_field(4)
    function_id: int = betterproto.uint32_field(5)
    cub_output_max: int = betterproto.uint32_field(6)
    flags: int = betterproto.uint32_field(7)
    callparameter: bytes = betterproto.bytes_field(8)
    ping_only: bool = betterproto.bool_field(9)
    max_outstanding_calls: int = betterproto.uint32_field(10)


@dataclass
class CMsgClientServiceModule(betterproto.Message):
    module_crc: int = betterproto.uint32_field(1)
    module_hash: bytes = betterproto.bytes_field(2)
    module_content: bytes = betterproto.bytes_field(3)


@dataclass
class CMsgClientServiceCallResponse(betterproto.Message):
    sysid_routing: bytes = betterproto.bytes_field(1)
    call_handle: int = betterproto.uint32_field(2)
    module_crc: int = betterproto.uint32_field(3)
    module_hash: bytes = betterproto.bytes_field(4)
    ecallresult: int = betterproto.uint32_field(5)
    result_content: bytes = betterproto.bytes_field(6)
    os_version_info: bytes = betterproto.bytes_field(7)
    system_info: bytes = betterproto.bytes_field(8)
    load_address: float = betterproto.fixed64_field(9)
    exception_record: bytes = betterproto.bytes_field(10)
    portable_os_version_info: bytes = betterproto.bytes_field(11)
    portable_system_info: bytes = betterproto.bytes_field(12)
    was_converted: bool = betterproto.bool_field(13)
    internal_result: int = betterproto.uint32_field(14)
    current_count: int = betterproto.uint32_field(15)
    last_call_handle: int = betterproto.uint32_field(16)
    last_call_module_crc: int = betterproto.uint32_field(17)
    last_call_sysid_routing: bytes = betterproto.bytes_field(18)
    last_ecallresult: int = betterproto.uint32_field(19)
    last_callissue_delta: int = betterproto.uint32_field(20)
    last_callcomplete_delta: int = betterproto.uint32_field(21)


@dataclass
class CMsgAMUnlockStreaming(betterproto.Message):
    pass


@dataclass
class CMsgAMUnlockStreamingResponse(betterproto.Message):
    eresult: int = betterproto.int32_field(1)
    encryption_key: bytes = betterproto.bytes_field(2)


@dataclass
class CMsgAMUnlockHEVC(betterproto.Message):
    pass


@dataclass
class CMsgAMUnlockHEVCResponse(betterproto.Message):
    eresult: int = betterproto.int32_field(1)


@dataclass
class CMsgClientPlayingSessionState(betterproto.Message):
    playing_blocked: bool = betterproto.bool_field(2)
    playing_app: int = betterproto.uint32_field(3)


@dataclass
class CMsgClientKickPlayingSession(betterproto.Message):
    only_stop_game: bool = betterproto.bool_field(1)


@dataclass
class CMsgClientVoiceCallPreAuthorize(betterproto.Message):
    caller_steamid: float = betterproto.fixed64_field(1)
    receiver_steamid: float = betterproto.fixed64_field(2)
    caller_id: int = betterproto.int32_field(3)
    hangup: bool = betterproto.bool_field(4)


@dataclass
class CMsgClientVoiceCallPreAuthorizeResponse(betterproto.Message):
    caller_steamid: float = betterproto.fixed64_field(1)
    receiver_steamid: float = betterproto.fixed64_field(2)
    eresult: int = betterproto.int32_field(3)
    caller_id: int = betterproto.int32_field(4)


@dataclass
class CMsgBadgeCraftedNotification(betterproto.Message):
    appid: int = betterproto.uint32_field(1)
    badge_level: int = betterproto.uint32_field(2)



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: steammessages_parental.proto
# plugin: python-betterproto

from dataclasses import dataclass
from typing import List

import betterproto


@dataclass
class ParentalApp(betterproto.Message):
    appid: int = betterproto.uint32_field(1)
    is_allowed: bool = betterproto.bool_field(2)


@dataclass
class ParentalSettings(betterproto.Message):
    steamid: float = betterproto.fixed64_field(1)
    applist_base_id: int = betterproto.uint32_field(2)
    applist_base_description: str = betterproto.string_field(3)
    applist_base: List["ParentalApp"] = betterproto.message_field(4)
    applist_custom: List["ParentalApp"] = betterproto.message_field(5)
    passwordhashtype: int = betterproto.uint32_field(6)
    salt: bytes = betterproto.bytes_field(7)
    passwordhash: bytes = betterproto.bytes_field(8)
    is_enabled: bool = betterproto.bool_field(9)
    enabled_features: int = betterproto.uint32_field(10)
    recovery_email: str = betterproto.string_field(11)
    is_site_license_lock: bool = betterproto.bool_field(12)


@dataclass
class CParental_EnableParentalSettings_Request(betterproto.Message):
    password: str = betterproto.string_field(1)
    settings: "ParentalSettings" = betterproto.message_field(2)
    sessionid: str = betterproto.string_field(3)
    enablecode: int = betterproto.uint32_field(4)
    steamid: float = betterproto.fixed64_field(10)


@dataclass
class CParental_EnableParentalSettings_Response(betterproto.Message):
    pass


@dataclass
class CParental_DisableParentalSettings_Request(betterproto.Message):
    password: str = betterproto.string_field(1)
    steamid: float = betterproto.fixed64_field(10)


@dataclass
class CParental_DisableParentalSettings_Response(betterproto.Message):
    pass


@dataclass
class CParental_GetParentalSettings_Request(betterproto.Message):
    steamid: float = betterproto.fixed64_field(10)


@dataclass
class CParental_GetParentalSettings_Response(betterproto.Message):
    settings: "ParentalSettings" = betterproto.message_field(1)


@dataclass
class CParental_GetSignedParentalSettings_Request(betterproto.Message):
    priority: int = betterproto.uint32_field(1)


@dataclass
class CParental_GetSignedParentalSettings_Response(betterproto.Message):
    serialized_settings: bytes = betterproto.bytes_field(1)
    signature: bytes = betterproto.bytes_field(2)


@dataclass
class CParental_SetParentalSettings_Request(betterproto.Message):
    password: str = betterproto.string_field(1)
    settings: "ParentalSettings" = betterproto.message_field(2)
    new_password: str = betterproto.string_field(3)
    sessionid: str = betterproto.string_field(4)
    steamid: float = betterproto.fixed64_field(10)


@dataclass
class CParental_SetParentalSettings_Response(betterproto.Message):
    pass


@dataclass
class CParental_ValidateToken_Request(betterproto.Message):
    unlock_token: str = betterproto.string_field(1)


@dataclass
class CParental_ValidateToken_Response(betterproto.Message):
    pass


@dataclass
class CParental_ValidatePassword_Request(betterproto.Message):
    password: str = betterproto.string_field(1)
    session: str = betterproto.string_field(2)
    send_unlock_on_success: bool = betterproto.bool_field(3)


@dataclass
class CParental_ValidatePassword_Response(betterproto.Message):
    token: str = betterproto.string_field(1)


@dataclass
class CParental_LockClient_Request(betterproto.Message):
    session: str = betterproto.string_field(1)


@dataclass
class CParental_LockClient_Response(betterproto.Message):
    pass


@dataclass
class CParental_RequestRecoveryCode_Request(betterproto.Message):
    pass


@dataclass
class CParental_RequestRecoveryCode_Response(betterproto.Message):
    pass


@dataclass
class CParental_DisableWithRecoveryCode_Request(betterproto.Message):
    recovery_code: int = betterproto.uint32_field(1)
    steamid: float = betterproto.fixed64_field(10)


@dataclass
class CParental_DisableWithRecoveryCode_Response(betterproto.Message):
    pass


@dataclass
class CParental_ParentalSettingsChange_Notification(betterproto.Message):
    serialized_settings: bytes = betterproto.bytes_field(1)
    signature: bytes = betterproto.bytes_field(2)
    password: str = betterproto.string_field(3)
    sessionid: str = betterproto.string_field(4)


@dataclass
class CParental_ParentalUnlock_Notification(betterproto.Message):
    password: str = betterproto.string_field(1)
    sessionid: str = betterproto.string_field(2)


@dataclass
class CParental_ParentalLock_Notification(betterproto.Message):
    sessionid: str = betterproto.string_field(1)



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: steammessages_unified_test.proto
# plugin: python-betterproto

from dataclasses import dataclass

import betterproto


@dataclass
class CMsgTest_MessageToClient_Request(betterproto.Message):
    some_text: str = betterproto.string_field(1)


@dataclass
class CMsgTest_MessageToClient_Response(betterproto.Message):
    some_text: str = betterproto.string_field(1)


@dataclass
class CMsgTest_NotifyClient_Notification(betterproto.Message):
    some_text: str = betterproto.string_field(1)


@dataclass
class CMsgTest_MessageToServer_Request(betterproto.Message):
    some_text: str = betterproto.string_field(1)


@dataclass
class CMsgTest_MessageToServer_Response(betterproto.Message):
    some_text: str = betterproto.string_field(1)


@dataclass
class CMsgTest_NotifyServer_Notification(betterproto.Message):
    some_text: str = betterproto.string_field(1)



# -*- coding: utf-8 -*-

"""
The MIT License (MIT)

Copyright (c) 2015-2020 Rapptz
Copyright (c) 2015 Rossen Georgiev <rossen@rgp.io>
Copyright (c) 2020 James

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.

EnumMeta from https://github.com/Rapptz/discord.py/blob/master/discord/enums.py
Enums from https://github.com/ValvePython/steam/blob/master/steam/enums/common.py
"""

from enum import EnumMeta as _EnumMeta, _is_dunder, _is_descriptor
from types import MappingProxyType
from typing import Any, Dict, List, Tuple

__all__ = (
    "Enum",
    "IntEnum",
    "EResult",
    "EUniverse",
    "EType",
    "ETypeChar",
    "EInstanceFlag",
    "EFriendRelationship",
    "EPersonaState",
    "EPersonaStateFlag",
    "ECommunityVisibilityState",
    "ETradeOfferState",
    "EChatEntryType",
    "EUIMode",
)


class EnumMember:
    _enum_cls_: "Enum"
    name: str
    value: Any

    def __new__(cls, **kwargs) -> "EnumMember":
        self = super().__new__(cls)
        try:
            self.name = kwargs["name"]
            self.value = kwargs["value"]
        except KeyError:
            pass
        finally:
            return self

    @classmethod
    def __call__(cls, *, name, value):
        return cls.__new__(cls, name=name, value=value)

    def __repr__(self):
        return f"<{self._enum_cls_.__name__}.{self.name}: {self.value!r}>"

    def __str__(self):
        return f"{self._enum_cls_.__name__}.{self.name}"

    def __hash__(self):
        return hash((self.name, self.value))

    def __eq__(self, other):
        try:
            if other._enum_cls_ is self._enum_cls_:
                return self.value == other.value
            return False
        except AttributeError:
            return NotImplemented


class IntEnumMember(EnumMember, int):
    _enum_cls_: "IntEnum"
    value: int

    def __new__(cls, **kwargs) -> "IntEnumMember":
        try:
            value = kwargs["value"]
            self = int.__new__(cls, value)
            self.name = kwargs["name"]
            self.value = value
            return self
        except KeyError:
            return int.__new__(cls)

    def __bool__(self):
        return True


class EnumMeta(type):
    def __new__(mcs, name: str, bases: Tuple[type, ...], attrs: Dict[str, Any]) -> "Enum":
        value_mapping: Dict[Any, EnumMember] = {}
        member_mapping: Dict[str, EnumMember] = {}
        member_names: List[str] = []

        try:
            value_cls = IntEnumMember(name=name) if IntEnum in bases else EnumMember(name=name)
        except NameError:
            value_cls = EnumMember(name=name)

        for key, value in tuple(attrs.items()):
            is_descriptor = _is_descriptor(value)
            if key[0] == "_" and not is_descriptor:
                continue

            # special case for classmethods to pass through
            if isinstance(value, classmethod):
                continue

            if is_descriptor:
                setattr(value_cls, key, value)
                del attrs[key]
                continue

            try:
                new_value = value_mapping[value]
            except KeyError:
                new_value = value_cls(name=key, value=value)
                value_mapping[value] = new_value
                member_names.append(key)

            member_mapping[key] = new_value
            attrs[key] = new_value

        attrs["_enum_value_map_"] = value_mapping
        attrs["_enum_member_map_"] = member_mapping
        attrs["_enum_member_names_"] = member_names
        enum_class: "Enum" = super().__new__(mcs, name, bases, attrs)
        for member in value_mapping.values():  # edit each value to ensure it's correct
            member._enum_cls_ = enum_class
        return enum_class

    def __call__(cls, value):
        if isinstance(value, cls):
            return value
        try:
            return cls._enum_value_map_[value]
        except (KeyError, TypeError):
            raise ValueError(f"{value!r} is not a valid {cls.__name__}")

    def __repr__(cls):
        return f"<enum {cls.__name__!r}>"

    def __iter__(cls):
        return (cls._enum_member_map_[name] for name in cls._enum_member_names_)

    def __reversed__(cls):
        return (cls._enum_member_map_[name] for name in reversed(cls._enum_member_names_))

    def __len__(cls):
        return len(cls._enum_member_names_)

    def __getitem__(cls, key):
        return cls._enum_member_map_[key]

    def __getattr__(cls, name):
        if _is_dunder(name):
            raise AttributeError(name)
        try:
            return cls._enum_value_map_[name]
        except KeyError:
            raise AttributeError(name) from None

    def __setattr__(cls, name, value):
        if name in cls._enum_member_map_:
            raise AttributeError(f"{cls.__name__}: cannot reassign Enum members.")
        super().__setattr__(name, value)

    def __delattr__(cls, attr):
        if attr in cls._enum_member_map_:
            raise AttributeError(f"{cls.__name__}: cannot delete Enum members.")
        super().__delattr__(attr)

    def __instancecheck__(self, instance):
        # isinstance(x, Y)
        # -> __instancecheck__(Y, x)
        try:
            cls = instance._enum_cls_
            return cls is self or issubclass(cls, self)
        except AttributeError:
            return False

    def __dir__(cls):
        return ["__class__", "__doc__", "__members__", "__module__"] + cls._enum_member_names_

    def __contains__(cls, member):
        if not isinstance(member, EnumMember):
            raise TypeError(
                "unsupported operand type(s) for 'in':"
                f" '{member.__class__.__qualname__}' and '{cls.__class__.__qualname__}'"
            )
        return member.name in cls._enum_member_map_

    def __bool__(self):
        return True

    @property
    def __members__(cls):
        return MappingProxyType(cls._enum_member_map_)


class Enum(metaclass=EnumMeta):
    """A general enumeration, emulates enum.Enum."""

    @classmethod
    def try_value(cls, value):
        try:
            return cls._enum_value_map_[value]
        except (KeyError, TypeError):
            return value


class IntEnum(int, Enum):
    """An enumeration where all the values are integers, emulates enum.IntEnum."""


def patched_instance_check(self: _EnumMeta, instance: Any) -> bool:
    if isinstance(instance, (EnumMeta, EnumMember)):
        return True

    return type.__instancecheck__(self, instance)


_EnumMeta.__instancecheck__ = patched_instance_check  # fake it till you make it


# fmt: off

class EResult(IntEnum):
    Invalid                         = 0
    OK                              = 1  #: Success
    Fail                            = 2  #: Generic failure
    NoConnection                    = 3  #: No/failed network connection
    InvalidPassword                 = 5  #: Password/ticket is invalid
    LoggedInElsewhere               = 6  #: Same user logged in elsewhere
    InvalidProtocolVersion          = 7
    InvalidParameter                = 8
    FileNotFound                    = 9
    Busy                            = 10  #: Called method busy - action not taken
    InvalidState                    = 11  #: Called object was in an invalid state
    InvalidName                     = 12
    InvalidEmail                    = 13
    DuplicateName                   = 14
    AccessDenied                    = 15
    Timeout                         = 16
    Banned                          = 17  #: VAC2 banned
    AccountNotFound                 = 18
    InvalidSteamID                  = 19
    ServiceUnavailable              = 20  #: The requested service is currently unavailable
    NotLoggedOn                     = 21
    Pending                         = 22  #: Request is pending (may be in process, or waiting on third party)
    EncryptionFailure               = 23
    InsufficientPrivilege           = 24
    LimitExceeded                   = 25  #: Too much of a good thing
    Revoked                         = 26  #: Access has been revoked (used for revoked guest passes)
    Expired                         = 27  #: License/Guest pass the user is trying to access is expired
    AlreadyRedeemed                 = 28  #: Guest pass has already been redeemed by account, cannot be acked again
    DuplicateRequest                = 29
    AlreadyOwned                    = 30  #: All the games in guest pass redemption request are already owned by the user
    IPNotFound                      = 31
    PersistFailed                   = 32  #: Failed to write change to the data store
    LockingFailed                   = 33  #: Failed to acquire access lock for this operation
    LogonSessionReplaced            = 34
    ConnectFailed                   = 35
    HandshakeFailed                 = 36
    IOFailure                       = 37
    RemoteDisconnect                = 38
    ShoppingCartNotFound            = 39
    Blocked                         = 40
    Ignored                         = 41
    NoMatch                         = 42
    AccountDisabled                 = 43
    ServiceReadOnly                 = 44
    AccountNotFeatured              = 45  #: Account doesn't have value, so this feature isn't available
    AdministratorOK                 = 46  #: Allowed to take this action, but only because requester is admin
    ContentVersion                  = 47  #: A Version mismatch in content transmitted within the Steam protocol
    TryAnotherCM                    = 48  #: The current CM can't service the user making a request, should try another
    PasswordRequiredToKickSession   = 49  #: You are already logged in elsewhere, this cached credential login has failed
    AlreadyLoggedInElsewhere        = 50  #: You are already logged in elsewhere, you must wait
    Suspended                       = 51  #: Long running operation (content download) suspended/paused
    Cancelled                       = 52  #: Operation canceled (typically by user content download)
    DataCorruption                  = 53  #: Operation canceled because data is ill formed or unrecoverable
    DiskFull                        = 54  #: Operation canceled - not enough disk space.
    RemoteCallFailed                = 55  #: An remote call or IPC call failed
    ExternalAccountUnlinked         = 57  #: External account (PSN, Facebook...) is not linked to a Steam account
    PSNTicketInvalid                = 58  #: PSN ticket was invalid
    ExternalAccountAlreadyLinked    = 59  #: External account (PSN, Facebook...) is already linked to some other account
    RemoteFileConflict              = 60  #: The sync cannot resume due to a conflict between the local and remote files
    IllegalPassword                 = 61  #: The requested new password is not legal
    SameAsPreviousValue             = 62  #: New value is the same as the old one (secret question and answer)
    AccountLogonDenied              = 63  #: Account login denied due to 2nd factor authentication failure
    CannotUseOldPassword            = 64  #: The requested new password is not legal
    InvalidLoginAuthCode            = 65  #: Account login denied due to auth code invalid
    HardwareNotCapableOfIPT         = 67
    IPTInitError                    = 68
    ParentalControlRestricted       = 69  #: Operation failed due to parental control restrictions for current user
    FacebookQueryError              = 70
    ExpiredLoginAuthCode            = 71  #: Account login denied due to auth code expired
    IPLoginRestrictionFailed        = 72
    VerifiedEmailRequired           = 74
    NoMatchingURL                   = 75
    BadResponse                     = 76  #: Parse failure, missing field, etc.
    RequirePasswordReEntry          = 77  #: The user cannot complete the action until they re-enter their password
    ValueOutOfRange                 = 78  #: The value entered is outside the acceptable range
    UnexpectedError                 = 79  #: Something happened that we didn't expect to ever happen
    Disabled                        = 80  #: The requested service has been configured to be unavailable
    InvalidCEGSubmission            = 81  #: The set of files submitted to the CEG server are not valid!
    RestrictedDevice                = 82  #: The device being used is not allowed to perform this action
    RegionLocked                    = 83  #: The action could not be complete because it is region restricted
    RateLimitExceeded               = 84  #: Temporary rate limit exceeded. different from k_EResultLimitExceeded
    LoginDeniedNeedTwoFactor        = 85  #: Need two-factor code to login
    ItemDeleted                     = 86  #: The thing we're trying to access has been deleted
    AccountLoginDeniedThrottle      = 87  #: Login attempt failed, try to throttle response to possible attacker
    TwoFactorCodeMismatch           = 88  #: Two factor code mismatch
    TwoFactorActivationCodeMismatch = 89  #: Activation code for two-factor didn't match
    NotModified                     = 91  #: Data not modified
    TimeNotSynced                   = 93  #: The time presented is out of range or tolerance
    SMSCodeFailed                   = 94  #: SMS code failure (no match, none pending, etc.)
    AccountActivityLimitExceeded    = 96  #: Too many changes to this account
    PhoneActivityLimitExceeded      = 97  #: Too many changes to this phone
    RefundToWallet                  = 98  #: Cannot refund to payment method, must use wallet
    EmailSendFailure                = 99  #: Cannot send an email
    NotSettled                      = 100  #: Can't perform operation till payment has settled
    NeedCaptcha                     = 101  #: Needs to provide a valid captcha
    GSLTDenied                      = 102  #: A game server login token owned by this token's owner has been banned
    GSOwnerDenied                   = 103  #: Game server owner is denied for other reason
    InvalidItemType                 = 104  #: The type of thing we were requested to act on is invalid
    IPBanned                        = 105  #: The ip address has been banned from taking this action
    GSLTExpired                     = 106  #: This token has expired from disuse; can be reset for use
    InsufficientFunds               = 107  #: User doesn't have enough wallet funds to complete the action
    TooManyPending                  = 108  #: There are too many of this thing pending already
    NoSiteLicensesFound             = 109  #: No site licenses found
    WGNetworkSendExceeded           = 110  #: The WG couldn't send a response because we exceeded max network send size
    AccountNotFriends               = 111
    LimitedUserAccount              = 112
    CantRemoveItem                  = 113


class EUniverse(IntEnum):
    Invalid  = 0
    Public   = 1
    Beta     = 2
    Internal = 3
    Dev      = 4
    Max      = 6

    def __str__(self):
        return self.name


class EType(IntEnum):
    Invalid        = 0
    Individual     = 1  #: Single user account
    Multiseat      = 2  #: Multiseat (e.g. cybercafe) account
    GameServer     = 3  #: Game server account
    AnonGameServer = 4  #: Anonymous game server account
    Pending        = 5
    ContentServer  = 6  #: Content server
    Clan           = 7
    Chat           = 8
    ConsoleUser    = 9  #: Fake SteamID for local PSN account on PS3 or Live account on 360, etc.
    AnonUser       = 10
    Max            = 11

    def __str__(self):
        return self.name


class ETypeChar(IntEnum):
    I = EType.Invalid
    U = EType.Individual
    M = EType.Multiseat
    G = EType.GameServer
    A = EType.AnonGameServer
    P = EType.Pending
    C = EType.ContentServer
    g = EType.Clan
    T = EType.Chat
    L = EType.Chat  #: Lobby/group chat, 'c' for clan chat
    c = EType.Chat  #: Clan chat
    a = EType.AnonUser

    def __str__(self):
        return self.name


class EInstanceFlag(IntEnum):
    MMSLobby = 0x20000
    Lobby    = 0x40000
    Clan     = 0x80000


class EFriendRelationship(IntEnum):
    NONE             = 0
    Blocked          = 1
    RequestRecipient = 2
    Friend           = 3
    RequestInitiator = 4
    Ignored          = 5
    IgnoredFriend    = 6
    SuggestedFriend  = 7
    Max              = 8


class EPersonaState(IntEnum):
    Offline        = 0
    Online         = 1
    Busy           = 2
    Away           = 3
    Snooze         = 4
    LookingToTrade = 5
    LookingToPlay  = 6
    Max            = 7

    def __str__(self):
        return self.name


class EPersonaStateFlag(IntEnum):
    NONE                 = 0
    HasRichPresence      = 1
    InJoinableGame       = 2
    Golden               = 4
    RemotePlayTogether   = 8
    ClientTypeWeb        = 256
    ClientTypeMobile     = 512
    ClientTypeTenfoot    = 1024
    ClientTypeVR         = 2048
    LaunchTypeGamepad    = 4096
    LaunchTypeCompatTool = 8192

    def __str__(self):
        return self.name


class ECommunityVisibilityState(IntEnum):
    NONE        = 0
    Private     = 1
    FriendsOnly = 2
    Public      = 3


class ETradeOfferState(IntEnum):
    Invalid                   = 1
    Active                    = 2
    Accepted                  = 3
    Countered                 = 4
    Expired                   = 5
    Canceled                  = 6
    Declined                  = 7
    InvalidItems              = 8
    ConfirmationNeed          = 9
    CanceledBySecondaryFactor = 10
    StateInEscrow             = 11


class EChatEntryType(IntEnum):
    Invalid          = 0
    ChatMsg          = 1  #: Normal text message from another user
    Typing           = 2  #: Another user is typing (not used in multi-user chat)
    InviteGame       = 3  #: Invite from other user into that users current game
    LeftConversation = 6  #: user has left the conversation ( closed chat window )
    Entered          = 7  #: User has entered the conversation (used in multi-user chat and group chat)
    WasKicked        = 8  #: user was kicked (data: 64-bit steamid of actor performing the kick)
    WasBanned        = 9  #: user was banned (data: 64-bit steamid of actor performing the ban)
    Disconnected     = 10  #: user disconnected
    HistoricalChat   = 11  #: a chat message from user's chat history or offline message
    LinkBlocked      = 14  #: a link was removed by the chat filter.


class EUIMode(IntEnum):
    Desktop    = 0
    BigPicture = 1
    Mobile     = 2
    Web        = 3


class EUserBadge(IntEnum):
    Invalid                           = 0
    YearsOfService                    = 1
    Community                         = 2
    Portal2PotatoARG                  = 3
    TreasureHunt                      = 4
    SummerSale2011                    = 5
    WinterSale2011                    = 6
    SummerSale2012                    = 7
    WinterSale2012                    = 8
    CommunityTranslator               = 9
    CommunityModerator                = 10
    ValveEmployee                     = 11
    GameDeveloper                     = 12
    GameCollector                     = 13
    TradingCardBetaParticipant        = 14
    SteamBoxBeta                      = 15
    Summer2014RedTeam                 = 16
    Summer2014BlueTeam                = 17
    Summer2014PinkTeam                = 18
    Summer2014GreenTeam               = 19
    Summer2014PurpleTeam              = 20
    Auction2014                       = 21
    GoldenProfile2014                 = 22
    TowerAttackMiniGame               = 23
    Winter2015ARG_RedHerring          = 24
    SteamAwards2016Nominations        = 25
    StickerCompletionist2017          = 26
    SteamAwards2017Nominations        = 27
    SpringCleaning2018                = 28
    Salien                            = 29
    RetiredModerator                  = 30
    SteamAwards2018Nominations        = 31
    ValveModerator                    = 32
    WinterSale2018                    = 33
    LunarNewYearSale2019              = 34
    LunarNewYearSale2019GoldenProfile = 35
    SpringCleaning2019                = 36
    Summer2019                        = 37
    Summer2019TeamHare                = 38
    Summer2019TeamTortoise            = 39
    Summer2019TeamCorgi               = 40
    Summer2019TeamCockatiel           = 41
    Summer2019TeamPig                 = 42
    SteamAwards2019Nominations        = 43
    WinterSaleEvent2019               = 44



# -*- coding: utf-8 -*-

"""
The MIT License (MIT)

Copyright (c) 2015-2020 Rapptz
Copyright (c) 2020 James

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.

This is a slightly modified version of discord.py's client
https://github.com/Rapptz/discord.py/blob/master/discord/client.py
"""

import asyncio
import logging
import signal
import sys
import traceback
from datetime import datetime
from typing import TYPE_CHECKING, Any, Awaitable, Callable, Dict, List, Optional, Tuple, Union

import aiohttp

from . import errors
from .abc import SteamID
from .clan import Clan
from .gateway import *
from .guard import generate_one_time_code
from .http import HTTPClient
from .iterators import TradesIterator
from .models import URL
from .state import ConnectionState
from .utils import ainput

if TYPE_CHECKING:
    import steam

    from .comment import Comment
    from .enums import EPersonaState, EUIMode
    from .game import Game
    from .group import Group
    from .invite import Invite
    from .protobufs import Msg, MsgProto
    from .trade import TradeOffer
    from .user import ClientUser, User


__all__ = ("Client",)

log = logging.getLogger(__name__)
EventType = Callable[..., Awaitable[None]]


def _cancel_tasks(loop: asyncio.AbstractEventLoop) -> None:
    tasks = asyncio.all_tasks(loop=loop)

    if not tasks:
        return

    log.info(f"Cleaning up after {len(tasks)} tasks.")
    for task in tasks:
        task.cancel()

    loop.run_until_complete(asyncio.gather(*tasks, return_exceptions=True))
    log.info("All tasks finished cancelling.")

    for task in tasks:
        if task.cancelled():
            continue
        if task.exception() is not None:
            loop.call_exception_handler(
                {
                    "message": "unhandled exception during Client.run shutdown.",
                    "exception": task.exception(),
                    "task": task,
                }
            )


def _cleanup_loop(loop: asyncio.AbstractEventLoop) -> None:
    try:
        _cancel_tasks(loop)
        loop.run_until_complete(loop.shutdown_asyncgens())
    finally:
        log.info("Closing the event loop.")
        loop.close()


class ClientEventTask(asyncio.Task):
    def __init__(
        self, original_coro: EventType, event_name: str, coro: Awaitable[None], *, loop: asyncio.AbstractEventLoop,
    ):
        super().__init__(coro, loop=loop)
        self.__event_name = event_name
        self.__original_coro = original_coro

    def __repr__(self):
        info = [
            ("state", self._state.lower()),
            ("event", self.__event_name),
            ("coro", repr(self.__original_coro)),
        ]
        if self._exception is not None:
            info.append(("exception", repr(self._exception)))
        return f'<ClientEventTask {" ".join(f"{t}={t!r}" for t in info)}>'


class Client:
    """Represents a client connection that connects to Steam.
    This class is used to interact with the Steam API and CMs.

    Parameters
    ----------
    loop: Optional[:class:`asyncio.AbstractEventLoop`]
        The :class:`asyncio.AbstractEventLoop` used for asynchronous operations.
        Defaults to ``None``, in which case the default event loop is used via
        :func:`asyncio.get_event_loop()`.

    Attributes
    -----------
    loop: :class:`asyncio.AbstractEventLoop`
        The event loop that the client uses for HTTP requests.
    ws:
        The connected websocket, this can be used to directly send messages
        to the connected CM.
    """

    def __init__(self, loop: Optional[asyncio.AbstractEventLoop] = None, **options):
        self.loop = loop or asyncio.get_event_loop()

        self.http = HTTPClient(loop=self.loop, client=self)
        self._connection = ConnectionState(loop=self.loop, client=self, http=self.http, **options)
        self.ws: Optional[SteamWebSocket] = None

        self.username: Optional[str] = None
        self.api_key: Optional[str] = None
        self.password: Optional[str] = None
        self.shared_secret: Optional[str] = None
        self.identity_secret: Optional[str] = None
        self.shared_secret: Optional[str] = None
        self.token: Optional[str] = None

        self._closed = True
        self._cm_list: Optional["CMServerList"] = None
        self._listeners: Dict[str, List[Tuple[asyncio.Future, Callable[..., bool]]]] = {}
        self._ready = asyncio.Event()

    @property
    def user(self) -> Optional["ClientUser"]:
        """Optional[:class:`~steam.ClientUser`]: Represents the connected client.
        ``None`` if not logged in."""
        return self.http.user

    @property
    def users(self) -> List["User"]:
        """List[:class:`~steam.User`]: Returns a list of all the users the account can see."""
        return self._connection.users

    @property
    def trades(self) -> List["TradeOffer"]:
        """List[:class:`~steam.TradeOffer`]: Returns a list of all the trades the connected client has seen."""
        return self._connection.trades

    @property
    def groups(self) -> List["Group"]:
        """List[:class:`~steam.Group`]: Returns a list of all the groups the connected client is in."""
        return self._connection.groups

    @property
    def clans(self) -> List["Clan"]:
        """List[:class:`~steam.Clan`]: Returns a list of all the clans the connected client is in."""
        return self._connection.clans

    async def code(self) -> str:
        """|coro|

        .. warning::
            Will wait for a Steam guard code using :func:`input` in an
            executor if no shared_secret is passed to :meth:`run` or :meth:`start`
            blocking exiting until one is entered.

        Returns
        -------
        :class:`str`
            The current steam guard code.
        """
        if self.shared_secret:
            return generate_one_time_code(self.shared_secret)
        code = await ainput("Please enter a Steam guard code\n>>> ")
        return code.strip()

    @property
    def latency(self) -> float:
        """:class:`float`: Measures latency between a HEARTBEAT and a HEARTBEAT_ACK in seconds."""
        return float("nan") if self.ws is None else self.ws.latency

    def event(self, coro: EventType) -> EventType:
        """A decorator that registers an event to listen to.

        The events must be a :ref:`coroutine <coroutine>`, if not, :exc:`TypeError` is raised.

        Usage: ::

            @client.event
            async def on_ready():
                print('Ready!')

        Raises
        --------
        :exc:`TypeError`
            The function passed is not a coroutine.
        """
        if not asyncio.iscoroutinefunction(coro):
            raise TypeError("Event registered must be a coroutine function")

        setattr(self, coro.__name__, coro)
        log.debug(f"{coro.__name__} has successfully been registered as an event")
        return coro

    async def _run_event(self, coro: EventType, event_name: str, *args, **kwargs) -> None:
        try:
            await coro(*args, **kwargs)
        except asyncio.CancelledError:
            pass
        except Exception as exc:
            try:
                await self.on_error(event_name, exc, *args, **kwargs)
            except asyncio.CancelledError:
                pass

    def _schedule_event(self, coro: EventType, event_name: str, *args, **kwargs) -> ClientEventTask:
        wrapped = self._run_event(coro, event_name, *args, **kwargs)
        # schedules the task
        return ClientEventTask(original_coro=coro, event_name=event_name, coro=wrapped, loop=self.loop)

    def dispatch(self, event: str, *args, **kwargs) -> None:
        method = f"on_{event}"

        try:
            coro = getattr(self, method)
        except AttributeError:
            return
        else:
            listeners = self._listeners.get(event)
            if listeners or event == "error":
                pass
            elif coro is None or coro.__code__.co_filename == __file__:
                # ignore events in this file that haven't been subclassed
                return

        log.debug(f"Dispatching event {event}")

        if listeners:
            removed: List[int] = []
            for i, (future, condition) in enumerate(listeners):
                if future.cancelled():
                    removed.append(i)
                    continue

                try:
                    result = condition(*args)
                except Exception as exc:
                    future.set_exception(exc)
                    removed.append(i)
                else:
                    if result:
                        if len(args) == 0:
                            future.set_result(None)
                        elif len(args) == 1:
                            future.set_result(args[0])
                        else:
                            future.set_result(args)
                        removed.append(i)

            if len(removed) == len(listeners):
                self._listeners.pop(event)
            else:
                for idx in reversed(removed):
                    del listeners[idx]

        self._schedule_event(coro, method, *args, **kwargs)

    def _handle_ready(self) -> None:
        self._ready.set()
        self.dispatch("ready")

    def is_ready(self) -> bool:
        """Specifies if the client's internal cache is ready for use."""
        return self._ready.is_set()

    def is_closed(self) -> bool:
        """Indicates if connection is closed to the API or CMs."""
        return self._closed

    def run(self, *args, **kwargs) -> None:
        """
        A blocking call that abstracts away the event loop
        initialisation from you.

        If you want more control over the event loop then this
        function should not be used. Use :meth:`start` coroutine
        or :meth:`login`.
        """
        loop = self.loop

        try:
            loop.add_signal_handler(signal.SIGINT, loop.stop)
            loop.add_signal_handler(signal.SIGTERM, loop.stop)
        except NotImplementedError:
            pass

        async def runner():
            try:
                await self.start(*args, **kwargs)
            finally:
                await self.close()

        def stop_loop_on_completion(_):
            loop.stop()

        future = asyncio.ensure_future(runner(), loop=loop)
        future.add_done_callback(stop_loop_on_completion)
        try:
            loop.run_forever()
        except KeyboardInterrupt:
            log.info("Received signal to terminate the client and event loop.")
        finally:
            future.remove_done_callback(stop_loop_on_completion)
            log.info("Cleaning up tasks.")
            _cleanup_loop(loop)

        if not future.cancelled():
            return future.result()

    async def login(
        self, username: str, password: str, api_key: Optional[str] = None, shared_secret: Optional[str] = None,
    ) -> None:
        """|coro|
        Logs in a Steam account and the Steam API with the specified credentials.

        Parameters
        ----------
        username: :class:`str`
            The username of the user's account.
        password: :class:`str`
            The password of the user's account.
        api_key: Optional[:class:`str`]
            The accounts api key for fetching info about the account.
            This can be left and the library will fetch it for you.
            If ``None`` is passed, one will be generated.
        shared_secret: Optional[:class:`str`]
            The shared_secret of the desired Steam account,
            used to generate the 2FA code for login. If ``None`` is passed,
            the code will need to be inputted by the user via :meth:`code`.

        Raises
        ------
        :exc:`TypeError`
            Unexpected keyword arguments were received.
        :exc:`.InvalidCredentials`
            The wrong credentials are passed.
        :exc:`.HTTPException`
            An unknown HTTP related error occurred.
        :exc:`.NoCMsFound`
            No community managers could be found to connect to.
        """
        log.info(f"Logging in to steamcommunity.com")
        self.api_key = api_key
        self.username = username
        self.password = password
        self.shared_secret = shared_secret

        await self.http.login(
            username=username, password=password, api_key=api_key, shared_secret=shared_secret,
        )

    async def close(self) -> None:
        """|coro|
        Closes the connection to Steam CMs and logs out.
        """
        if self._closed:
            return
        if self.ws is not None:
            await self.ws.close()
        await self.http.logout()
        await self.http._session.close()
        self._closed = True
        self._ready.clear()

    def clear(self) -> None:
        """Clears the internal state of the bot.
        After this, the bot can be considered "re-opened", i.e. :meth:`is_closed`
        and :meth:`is_ready` both return ``False``. This also clears the internal
        cache.
        """
        self._closed = False
        self._ready.clear()
        self._connection.clear()
        self.http.recreate()

    async def start(self, *args, **kwargs) -> None:
        """|coro|
        A shorthand coroutine for :meth:`login` and :meth:`connect`.
        If no shared_secret is passed, you will have to manually
        enter a Steam guard code.

        Raises
        ------
        TypeError
            Unexpected keyword arguments were received.
        :exc:`.InvalidCredentials`
            The wrong credentials are passed.
        :exc:`.HTTPException`
            An unknown HTTP related error occurred.
        :exc:`.NoCMsFound`
            No community managers could be found to connect to.
        """
        self.username = username = kwargs.pop("username", None) or args[0]
        self.password = password = kwargs.pop("password", None) or args[1]
        self.api_key = api_key = kwargs.pop("api_key", None)
        self.shared_secret = shared_secret = kwargs.pop("shared_secret", None)
        self.identity_secret = identity_secret = kwargs.pop("identity_secret", None)

        if identity_secret is None:
            log.info("Trades will not be automatically accepted when sent as no identity_secret was passed.")
        if not (username or password):
            raise errors.LoginError("one or more required login detail is missing")
        if kwargs:
            raise TypeError(f"unexpected keyword argument(s) {list(kwargs.keys())}")

        await self.login(
            username=username, password=password, api_key=api_key, shared_secret=shared_secret,
        )
        self.loop.create_task(self._connection.__ainit__())
        self._closed = False
        await self.connect()

    async def _connect(self) -> None:
        resp = await self.http.request("GET", url=f"{URL.COMMUNITY}/chat/clientjstoken")
        if not resp["logged_in"]:  # we got logged out :(
            await self.login(self.username, self.password, self.api_key, self.shared_secret)
            await self._connect()
        self.token = resp["token"]
        coro = SteamWebSocket.from_client(self, cms=self._cm_list)
        self.ws: SteamWebSocket = await asyncio.wait_for(coro, timeout=60)
        while 1:
            await self.ws.poll_event()

    async def connect(self) -> None:
        """|coro|
        Initialize a connection to a Steam CM.
        """
        while not self.is_closed():
            try:
                await self._connect()
            except (
                OSError,
                aiohttp.ClientError,
                asyncio.TimeoutError,
                errors.HTTPException,
            ):
                self.dispatch("disconnect")
            except ConnectionClosed as exc:
                self._cm_list = exc.cm_list
                self.dispatch("disconnect")
            finally:
                if self.is_closed():
                    return

                log.info(f"Attempting to connect to another CM")
                await asyncio.sleep(5)

    # state stuff

    def get_user(self, *args, **kwargs) -> Optional["User"]:
        r"""Returns a user from cache with a matching ID.

        Parameters
        ----------
        \*args
            The arguments to pass to :meth:`~steam.utils.make_steam64`.
        \*\*kwargs
            The keyword arguments to pass to :meth:`~steam.utils.make_steam64`.

        Returns
        -------
        Optional[:class:`~steam.User`]
            The user or ``None`` if the user was not found.
        """
        steam_id = SteamID(*args, **kwargs)
        return self._connection.get_user(steam_id.id64)

    async def fetch_user(self, *args, **kwargs) -> Optional["User"]:
        r"""|coro|
        Fetches a user from the API with a matching ID.

        Parameters
        ----------
        \*args
            The arguments to pass to :meth:`~steam.utils.make_steam64`.
        \*\*kwargs
            The keyword arguments to pass to :meth:`~steam.utils.make_steam64`.

        Returns
        -------
        Optional[:class:`~steam.User`]
            The user or ``None`` if the user was not found.
        """
        steam_id = SteamID(*args, **kwargs)
        return await self._connection.fetch_user(steam_id.id64)

    async def fetch_users(self, *ids: List[int]) -> List[Optional["User"]]:
        r"""|coro|
        Fetches a list of :class:`~steam.User` from their IDs
        from the API with a matching ID. The :class:`~steam.User` objects returned
        are unlikely to retain the order they were originally in.

        Parameters
        ----------
        \*ids: List[:class:`int`]
            The user's IDs.

        Returns
        -------
        List[Optional[:class:`~steam.User`]]
            A list of the users or ``None`` if the user was not found.
        """
        steam_ids = [SteamID(id).id64 for id in ids]
        return await self._connection.fetch_users(steam_ids)

    async def fetch_user_named(self, name: str) -> Optional["User"]:
        """|coro|
        Fetches a user from https://steamcommunity.com with a matching name.

        Parameters
        ----------
        name: :class:`str`
            The name of the Steam user.

        Returns
        -------
        Optional[:class:`~steam.User`]
            The user or ``None`` if the user was not found.
        """
        steam_id = await SteamID.from_url(f"{URL.COMMUNITY}/id/{name}", self.http._session)
        if not steam_id:
            return None
        return await self.fetch_user(id=steam_id.id64)

    def get_trade(self, id: int) -> Optional["TradeOffer"]:
        """Get a trade from cache with a matching ID.

        Parameters
        ----------
        id: :class:`int`
            The id of the trade to search for from the cache.

        Returns
        -------
        Optional[:class:`~steam.TradeOffer`]
            The trade offer or ``None`` if the trade was not found.
        """
        return self._connection.get_trade(id)

    async def fetch_trade(self, id: int) -> Optional["TradeOffer"]:
        """|coro|
        Fetches a trade from the API with a matching ID.

        Parameters
        ----------
        id: :class:`int`
            The ID of the trade to search for from the API.

        Returns
        -------
        Optional[:class:`~steam.TradeOffer`]
            The trade offer or ``None`` if the trade was not found.
        """
        return await self._connection.fetch_trade(id)

    def get_group(self, id: int) -> Optional["Group"]:
        """Get a group from cache with a matching ID.

        Parameters
        ----------
        id: :class:`int`
            The id of the group to search for from the cache.

        Returns
        -------
        Optional[:class:`~steam.Group`]
            The group or ``None`` if the group was not found.
        """
        return self._connection.get_group(id)

    def get_clan(self, *args, **kwargs) -> Optional["Clan"]:
        """Get a clan from cache with a matching ID.

        Parameters
        ----------
        \*args
            The arguments to pass to :meth:`~steam.utils.make_steam64`.
        \*\*kwargs
            The keyword arguments to pass to :meth:`~steam.utils.make_steam64`.

        Returns
        -------
        Optional[:class:`~steam.Clan`]
            The clan or ``None`` if the clan was not found.
        """
        kwargs["is_clan"] = True
        steam_id = SteamID(*args, **kwargs)
        return self._connection.get_clan(steam_id.id)

    async def fetch_clan(self, *args, **kwargs) -> Optional["Clan"]:
        r"""|coro|
        Fetches a clan from the websocket with a matching ID.

        Parameters
        ----------
        \*args
            The arguments to pass to :meth:`~steam.utils.make_steam64`.
        \*\*kwargs
            The keyword arguments to pass to :meth:`~steam.utils.make_steam64`.

        Returns
        -------
        Optional[:class:`~steam.Clan`]
            The clan or ``None`` if the clan was not found.
        """
        kwargs["is_clan"] = True
        steam_id = SteamID(*args, **kwargs)
        return await self._connection.fetch_clan(steam_id.id64)

    async def fetch_clan_named(self, name: str) -> Optional["Clan"]:
        """|coro|
        Fetches a clan from https://steamcommunity.com with a matching name.

        Parameters
        ----------
        name: :class:`str`
            The name of the Steam clan.

        Returns
        -------
        Optional[:class:`~steam.Clan`]
            The clan or ``None`` if the clan was not found.
        """
        steam_id = await SteamID.from_url(f"{URL.COMMUNITY}/clans/{name}", self.http._session)
        if steam_id is None:
            return None
        return await self._connection.fetch_clan(steam_id.id64)

    def trade_history(
        self,
        limit: Optional[int] = 100,
        before: Optional[datetime] = None,
        after: Optional[datetime] = None,
        active_only: bool = False,
    ) -> TradesIterator:
        """An :class:`~steam.iterators.AsyncIterator` for accessing a
        :class:`ClientUser`'s :class:`~steam.TradeOffer` objects.

        Examples
        -----------

        Usage: ::

            async for trade in client.trade_history(limit=10):
                print('Partner:', trade.partner, 'Sent:')
                print(', '.join(item.name if item.name else str(item.asset_id) for item in trade.items_to_receive)
                      if trade.items_to_receive else 'Nothing')

        Flattening into a list: ::

            trades = await client.trade_history(limit=50).flatten()
            # trades is now a list of TradeOffer

        All parameters are optional.

        Parameters
        ----------
        limit: Optional[:class:`int`]
            The maximum number of trades to search through.
            Default is 100 which will fetch the first 100 trades.
            Setting this to ``None`` will fetch all of the user's trades,
            but this will be a very slow operation.
        before: Optional[:class:`datetime.datetime`]
            A time to search for trades before.
        after: Optional[:class:`datetime.datetime`]
            A time to search for trades after.
        active_only: Optional[:class:`bool`]
            Whether or not to fetch only active trades defaults to ``True``.

        Yields
        ---------
        :class:`~steam.TradeOffer`
        """
        return TradesIterator(state=self._connection, limit=limit, before=before, after=after, active_only=active_only,)

    # misc

    async def change_presence(
        self,
        *,
        game: Optional["Game"] = None,
        games: Optional[List["Game"]] = None,
        state: Optional["EPersonaState"] = None,
        ui_mode: Optional["EUIMode"] = None,
        force_kick: bool = False,
    ) -> None:
        """|coro|
        Set your status.

        Parameters
        ----------
        game: :class:`~steam.Game`
            A games to set your status as.
        games: List[:class:`~steam.Game`]
            A list of games to set your status to.
        state: :class:`~steam.EPersonaState`
            The state to show your account as.

            .. note::
                Setting your status to :attr:`~steam.EPersonaState.Offline`,
                will stop you receiving persona state updates and by extension
                :meth:`on_user_update` will stop being dispatched.

        ui_mode: :class:`~steam.EUIMode`
            The UI mode to set your status to.
        force_kick: :class:`bool`
            Whether or not to forcefully kick any other playing sessions.
        """
        games = [game.to_dict() for game in games] if games is not None else []
        if game is not None:
            games.append(game.to_dict())
        await self.ws.change_presence(games=games, state=state, ui_mode=ui_mode, force_kick=force_kick)

    async def wait_until_ready(self) -> None:
        """|coro|
        Waits until the client's internal cache is all ready.
        """
        await self._ready.wait()

    def wait_for(
        self, event: str, *, check: Optional[Callable[..., bool]] = None, timeout: Optional[float] = None
    ) -> Awaitable[Any]:
        """|coro|
        Waits for an event to be dispatched.

        The ``timeout`` parameter is passed to :func:`asyncio.wait_for`. By default,
        it does not timeout. Note that this does propagate the :exc:`asyncio.TimeoutError`
        for you in case of timeout and is provided for ease of use.
        In case the event returns multiple arguments, a :class:`tuple` containing those
        arguments is returned instead. Please check the
        `documentation <https://steampy.rtfd.io/en/latest/api.html#event-reference>`_
        for a list of events and their parameters.

        .. note::
            This function returns the **first event that meets the requirements**.

        Parameters
        ------------
        event: :class:`str`
            The event name, similar to the
            `event reference <https://steampy.rtfd.io/en/latest/api.html#event-reference>`_,
            but without the ``on_`` prefix, to wait for.
        check: Optional[Callable[..., :class:`bool`]]
            A predicate to check what to wait for. The arguments must meet the
            parameters of the event being waited for.
            The check **MUST** return a :class:`bool`.
        timeout: Optional[:class:`float`]
            The number of seconds to wait before timing out and raising
            :exc:`asyncio.TimeoutError`.

        Raises
        -------
        asyncio.TimeoutError
            If the provided timeout was reached.

        Returns
        --------
        Any
            Returns no arguments, a single argument, or a :class:`tuple` of multiple
            arguments that mirrors the parameters passed in the
            `event reference <https://steampy.rtfd.io/en/latest/api.html#event-reference>`_.
        """
        future = self.loop.create_future()
        check = check if check is not None else lambda *_: True

        event_lower = event.lower()
        try:
            listeners = self._listeners[event_lower]
        except KeyError:
            listeners = []
            self._listeners[event_lower] = listeners

        listeners.append((future, check))
        return asyncio.wait_for(future, timeout)

    # events to be subclassed

    async def on_connect(self):
        """|coro|
        Called when the client has successfully connected to Steam. This is not
        the same as the client being fully prepared, see :func:`on_ready` for that.

        The warnings on :meth:`on_ready` also apply.
        """

    async def on_disconnect(self):
        """|coro|
        Called when the client has disconnected from Steam. This could happen either through
        the internet disconnecting, an explicit call to logout, or Steam terminating the connection.

        This function can be called multiple times.
        """

    async def on_ready(self):
        """|coro|
        Called after a successful login and the client has handled setting up
        trade and notification polling, along with setup the confirmation manager.

        .. note::
            In future this will be called when the client is done preparing the data received from Steam.
            Usually after login to a CM is successful.

        .. warning::

            This function is not guaranteed to be the first event called.
            Likewise, this function is **not** guaranteed to only be called
            once. This library implements reconnection logic and will therefore
            end up calling this event whenever a RESUME request fails.
        """

    async def on_login(self):
        """|coro|
        Called when the client has logged into https://steamcommunity.com and
        the :class:`~steam.ClientUser` is setup along with its friends list.
        """

    async def on_error(self, event: str, error: Exception, *args, **kwargs):
        r"""|coro|
        The default error handler provided by the client.

        Usually when an event raises an uncaught exception, a traceback is
        printed to stderr and the exception is ignored. If you want to
        change this behaviour and handle the exception for whatever reason
        yourself, this event can be overridden. Which, when done, will
        suppress the default action of printing the traceback.

        If you want exception to propagate out of the :class:`Client` class
        you can define an ``on_error`` handler consisting of a single empty
        :ref:`py:raise`. Exceptions raised by ``on_error`` will not be
        handled in any way by :class:`Client`.

        Parameters
        ----------
        event: :class:`str`
            The name of the event that errored.
        error: :exc:`Exception`
            The error that was raised.
        \*args:
            The positional arguments associated with the event.
        \*\*kwargs:
            The key-word arguments associated with the event.
        """
        print(f"Ignoring exception in {event}", file=sys.stderr)
        traceback.print_exception(type(error), error, error.__traceback__, file=sys.stderr)

    async def on_message(self, message: "steam.Message"):
        """|coro|
        Called when a message is created.

        Parameters
        ----------
        message: :class:`~steam.Message`
            The message that was received.
        """

    async def on_typing(self, user: "steam.User", when: datetime):
        """|coro|
        Called when typing is started.

        Parameters
        ----------
        user: :class:`~steam.User`
            The user that started typing.
        when: :class:`datetime.datetime`
            The time the user started typing at.
        """

    async def on_trade_receive(self, trade: "steam.TradeOffer"):
        """|coro|
        Called when the client receives a trade offer.

        Parameters
        ----------
        trade: :class:`~steam.TradeOffer`
            The trade offer that was received.
        """

    async def on_trade_send(self, trade: "steam.TradeOffer"):
        """|coro|
        Called when the client sends a trade offer.

        Parameters
        ----------
        trade: :class:`~steam.TradeOffer`
            The trade offer that was sent.
        """

    async def on_trade_accept(self, trade: "steam.TradeOffer"):
        """|coro|
        Called when the client or the trade partner accepts a trade offer.

        Parameters
        ----------
        trade: :class:`~steam.TradeOffer`
            The trade offer that was accepted.
        """

    async def on_trade_decline(self, trade: "steam.TradeOffer"):
        """|coro|
        Called when the client or the trade partner declines a trade offer.

        Parameters
        ----------
        trade: :class:`~steam.TradeOffer`
            The trade offer that was declined.
        """

    async def on_trade_cancel(self, trade: "steam.TradeOffer"):
        """|coro|
        Called when the client or the trade partner cancels a trade offer.

        .. note::
            This is called when the trade state becomes
            :attr:`~steam.ETradeOfferState.Canceled` and
            :attr:`~steam.ETradeOfferState.CanceledBySecondaryFactor`.

        Parameters
        ----------
        trade: :class:`~steam.TradeOffer`
            The trade offer that was cancelled.
        """

    async def on_trade_expire(self, trade: "steam.TradeOffer"):
        """|coro|
        Called when a trade offer expires due to being active for too long.

        Parameters
        ----------
        trade: :class:`~steam.TradeOffer`
            The trade offer that expired.
        """

    async def on_trade_counter(self, trade: "steam.TradeOffer"):
        """|coro|
        Called when the client or the trade partner counters a trade offer.

        Parameters
        ----------
        trade: :class:`~steam.TradeOffer`
            The trade offer that was countered.
        """

    async def on_comment(self, comment: "steam.Comment"):
        """|coro|
        Called when the client receives a comment notification.

        Parameters
        ----------
        comment: :class:`~steam.Comment`
            The comment received.
        """

    async def on_user_invite(self, invite: "steam.UserInvite"):
        """|coro|
        Called when the client receives an invite from a :class:`~steam.User`
        to become a friend.

        Parameters
        ----------
        invite: :class:`~steam.UserInvite`
            The invite received.
        """

    async def on_clan_invite(self, invite: "steam.ClanInvite"):
        """|coro|
        Called when the client receives an invite from a :class:`~steam.User`
        to join a Clan.

        Parameters
        ----------
        invite: :class:`~steam.ClanInvite`
            The invite received.
        """

    async def on_user_update(self, before: "steam.User", after: "steam.User"):
        """|coro|
        Called when a user's their state, due to one or more
        of the following attributes changing:

            - name
            - state
            - flags
            - avatar
            - last_logon
            - last_logout
            - last_seen_online
            - game


        Parameters
        ----------
        before: :class:`~steam.User`
            The user's state before.
        after: :class:`~steam.User`
            The user's state now.
        """

    async def on_socket_receive(self, msg: Union["Msg", "MsgProto"]):
        """|coro|
        Called when the connected web-socket parses a received
        ``Msg``/``MsgProto``

        Parameters
        ----------
        msg: Union[Msg, MsgProto]
            The received message.
        """

    async def on_socket_raw_receive(self, message: bytes):
        """|coro|
        Called when the connected web-socket receives
        raw :class:`bytes`. This isn't likely to be very useful.

        Parameters
        ----------
        message: bytes
            The raw received message.
        """

    async def on_socket_send(self, msg: Union["Msg", "MsgProto"]):
        """|coro|
        Called when the client sends a parsed ``Msg``/``MsgProto``
        to the connected web-socket.

        Parameters
        ----------
        msg: Union[Msg, MsgProto]
            The sent message.
        """

    async def on_socket_raw_send(self, message: bytes):
        """|coro|
        Called when the client sends raw :class:`bytes`
        to the connected web-socket.
        This isn't likely to be very useful.

        Parameters
        ----------
        message: bytes
            The raw sent message.
        """



# -*- coding: utf-8 -*-

"""
MIT License

Copyright (c) 2015 Rossen Georgiev <rossen@rgp.io>
Copyright (c) 2020 James

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.

This is an updated version of https://github.com/ValvePython/steam/tree/master/steam/core/msg
"""

from dataclasses import dataclass
from typing import Generic, Optional, Type, TypeVar, Union

import betterproto

from ..enums import EnumMember
from .emsg import *
from .headers import *
from .protobufs import *
from .unified import *

T = TypeVar("T", bound=betterproto.Message)
AllowedHeaders = (ExtendedMsgHdr, MsgHdrProtoBuf)
betterproto.Message.__bool__ = lambda self: bool(self.to_dict(include_default_values=False))


@dataclass
class FailedToParse(betterproto.Message):
    body: str = "!!! Failed To Parse !!!"


def get_cmsg(emsg: Union[EMsg, int]) -> Optional[Type[betterproto.Message]]:
    """Get a protobuf from its EMsg.

    Parameters
    ----------
    emsg: Union[:class:`EMsg`, :class:`int`]
        The EMsg of the protobuf.

    Returns
    -------
    The uninitialized protobuf.
    """
    return PROTOBUFS.get(EMsg.try_value(emsg))


def get_um(method_name: str) -> Optional[Type[betterproto.Message]]:
    """Get the protobuf for a certain Unified Message.

    Parameters
    ----------
    method_name: :class:`str`
        The name of the UM.

    Returns
    -------
    The uninitialized protobuf.
    """
    return UMS.get(method_name)


class MsgBase(Generic[T]):
    __slots__ = ("header", "proto", "body", "payload", "skip")

    def __init__(self, msg: EMsg, data: bytes, parse: bool, **kwargs):
        self.msg = EMsg.try_value(msg)
        self.body: Optional[T] = None
        self.payload: Optional[bytes] = None
        self.header: Union[AllowedHeaders, MsgHdr]

        if data:
            self.payload = data[self.skip :]
        if parse:
            self.parse()
        if kwargs:
            for (key, value) in kwargs.items():
                if isinstance(value, EnumMember):
                    kwargs[key] = value.value
            self.body.from_dict(kwargs)

    def __repr__(self):
        attrs = (
            "msg",
            "header",
        )
        resolved = [f"{attr}={getattr(self, attr)!r}" for attr in attrs]
        if isinstance(self.body, betterproto.Message):
            resolved.extend(f"{k}={v!r}" for k, v in self.body.to_dict(betterproto.Casing.SNAKE).items())
        return " ".join(resolved)

    def parse(self, proto: Type[T]) -> None:
        """Parse the payload/data into a protobuf."""
        if proto:
            self.body = proto()
            if self.payload:
                self.body = proto().parse(self.payload)
        else:
            self.body = FailedToParse

    @property
    def msg(self) -> Union[EMsg, int]:
        """Union[:class:`EMsg`, :class:`int`]: The :attr:`header`'s EMsg."""
        return self.header.msg

    @msg.setter
    def msg(self, value) -> None:
        self.header.msg = EMsg.try_value(value)

    @property
    def steam_id(self) -> Optional[int]:
        """Optional[:class:`int`]: The :attr:`header`'s 64 bit Steam ID."""
        return self.header.steam_id if isinstance(self.header, AllowedHeaders) else None

    @steam_id.setter
    def steam_id(self, value) -> None:
        if isinstance(self.header, AllowedHeaders):
            self.header.steam_id = value

    @property
    def session_id(self) -> Optional[int]:
        """Optional[:class:`int`]: The :attr:`header`'s session ID."""
        return self.header.session_id if isinstance(self.header, AllowedHeaders) else None

    @session_id.setter
    def session_id(self, value) -> None:
        if isinstance(self.header, AllowedHeaders):
            self.header.session_id = value

    def __bytes__(self):
        return bytes(self.header) + bytes(self.body)


class Msg(MsgBase[T]):
    r"""A wrapper around received protobuf messages.

    .. container:: operations

        .. describe:: bytes(x)

            Returns the sterilised message.

        .. describe:: x[y]

            Allows for type hinting of the messages
            body. e.g.::

                async def handle_multi(self, msg: MsgProto['CMsgMulti']) -> None:
                    log.debug('Received a multi, unpacking')
                    if msg.body.size_unzipped:  # this is now properly typed
                        ...

    Parameters
    ----------
    msg: :class:`EMsg`
        The emsg for the message.
    data: Optional[:class:`bytes`]
        The raw data for the message.
    extended: :class:`bool`
        Which header type to use, ``True`` uses
        :class:`.ExtendedMsgHdr` else it's :class:`.MsgHdr`.
    parse: :class:`bool`
        Whether or not to parse the data into a constructed protobuf.
    \*\*kwargs
        Any keyword-arguments to construct the :attr:`body` with.

    Attributes
    ----------
    header: Union[:class:`.ExtendedMsgHdr`, :class:`.MsgHdr`]
        The message's header.
    msg: :class:`EMsg`
        The emsg for the message.
    body
        The instance of the protobuf.
    payload: :class:`bytes`
        The raw data for the message.
    """

    def __init__(
        self, msg: EMsg, data: bytes = None, extended: bool = False, parse: bool = True, **kwargs,
    ):
        self.header = ExtendedMsgHdr(data) if extended else MsgHdr(data)
        self.proto = False
        self.skip = self.header.SIZE
        super().__init__(msg, data, parse, **kwargs)

    def __repr__(self):
        return f"<Msg {super().__repr__()}>"

    def parse(self):
        """Parse the payload/data into a protobuf."""
        if self.body is None:
            proto = get_cmsg(self.msg)
            super().parse(proto)


class MsgProto(MsgBase[T]):
    r"""A wrapper around received protobuf messages.

    .. container:: operations

        .. describe:: bytes(x)

            Returns the sterilised message.

        .. describe:: x[y]

            Allows for type hinting of the messages
            body. e.g.::

                async def handle_multi(self, msg: MsgProto['CMsgMulti']) -> None:
                    log.debug('Received a multi, unpacking')
                    if msg.body.size_unzipped:  # this is now properly typed
                        ...

    Parameters
    ----------
    msg: :class:`EMsg`
        The emsg for the message.
    data: Optional[:class:`bytes`]
        The raw data for the message.
    um_name: Optional[:class:`str`]
        The name of the Unified Message the protobuf is associated.
    parse: :class:`bool`
        Whether or not to parse the data into a constructed protobuf.
    \*\*kwargs
        Any keyword-arguments to construct the :attr:`body` with.

    Attributes
    ----------
    header: Union[:class:`.ExtendedMsgHdr`, :class:`.MsgHdr`]
        The message's header.
    msg: :class:`EMsg`
        The emsg for the message.
    body
        The instance of the protobuf.
    payload: :class:`bytes`
        The raw data for the message.
    """

    __slots__ = ("um_name",)

    def __init__(
        self, msg: EMsg, data: bytes = None, parse: bool = True, um_name: str = None, **kwargs,
    ):
        self.header = MsgHdrProtoBuf(data)
        self.skip = self.header._full_size
        self.proto = True
        self.um_name = um_name
        super().__init__(msg, data, parse, **kwargs)

    def __repr__(self):
        return f"<MsgProto {super().__repr__()}>"

    def parse(self):
        """Parse the payload/data into a protobuf."""
        if self.body is None:
            if self.msg in (
                EMsg.ServiceMethod,
                EMsg.ServiceMethodResponse,
                EMsg.ServiceMethodSendToClient,
                EMsg.ServiceMethodCallFromClient,
            ):
                name = self.header.job_name_target or self.um_name
                proto = get_um(name)
                if not name.endswith("_Response") and proto is None:
                    proto = get_um(f"{name}_Response")  # assume its a response
                if name:
                    self.header.job_name_target = name.replace("_Request", "").replace("_Response", "")

            else:
                proto = get_cmsg(self.msg)

            super().parse(proto)



# -*- coding: utf-8 -*-

"""
The MIT License (MIT)

Copyright (c) 2020 James

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"""

from typing import TYPE_CHECKING, Dict, Optional

from .cog import Cog
from .command import Command

if TYPE_CHECKING:
    from .context import Context

__all__ = ("HelpCommand",)


class HelpCommand(Command):
    """The default implementation of the help command."""

    def __init__(self):
        super().__init__(self.command_callback, name="help", help="Shows this message.")
        self.context: Optional["Context"] = None

    def __repr__(self):
        return "<default help-command>"

    def _get_doc(self, command: "Command") -> str:
        try:
            return command.help.splitlines()[0]
        except (IndexError, AttributeError):
            return ""

    async def _parse_arguments(self, ctx):
        # Make the parser think we don't have a cog so it doesn't
        # inject the parameter into `ctx.args`.
        original_cog = self.cog
        self.cog = None
        try:
            await super()._parse_arguments(ctx)
        finally:
            self.cog = original_cog

    async def command_callback(self, ctx: "Context", *, command: str = None) -> None:
        """The actual implementation of the help command."""
        self.context = ctx
        bot = ctx.bot
        if command is None:
            mapping = self.get_bot_mapping()
            return await self.send_help(mapping)
        # Check if it's a cog
        cog = bot.get_cog(command.capitalize())
        if cog is not None:
            return await self.send_cog_help(cog)
        command_ = bot.get_command(command)
        if command_ is not None:
            return await self.send_command_help(command_)

        await self.command_not_found(command)

    def get_bot_mapping(self) -> Dict[Optional["Cog"], Command]:
        bot = self.context.bot
        mapping = {name: cog.__cog_commands__ for (name, cog) in bot.__cogs__.items()}
        mapping[None] = [c for c in bot.commands if c not in mapping.values()]
        return mapping

    async def send_help(self, mapping: Dict[Optional["Cog"], Command]):
        message = []
        for name, commands in sorted(mapping.items()):
            if name is not None:
                message.append(f"--= {name}'s commands =--")
            else:
                message.append("--= Un-categorized commands =--")
            for command in commands:
                message.append(f'{command.name}{f": {self._get_doc(command)}" if command.help else ""}')
        await self.context.send("\n".join(message))

    async def send_cog_help(self, cog: "Cog"):
        message = [f"--= {cog.qualified_name}'s commands =--"]
        for name, command in sorted(cog.__commands__.items()):
            message.append(f'{name}{f": {self._get_doc(command)}" if command.help else ""}')
        await self.context.send("\n".join(message))

    async def send_command_help(self, command: "Command"):
        await self.context.send(f"Help with {command.name}:\n\n{command.help}")

    async def command_not_found(self, command: str):
        await self.context.send(f'The command "{command}" was not found.')



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: steammessages_cloud.proto
# plugin: python-betterproto

from dataclasses import dataclass
from typing import List

import betterproto


@dataclass
class CCloud_GetUploadServerInfo_Request(betterproto.Message):
    appid: int = betterproto.uint32_field(1)


@dataclass
class CCloud_GetUploadServerInfo_Response(betterproto.Message):
    server_url: str = betterproto.string_field(1)


@dataclass
class CCloud_BeginHTTPUpload_Request(betterproto.Message):
    appid: int = betterproto.uint32_field(1)
    file_size: int = betterproto.uint32_field(2)
    filename: str = betterproto.string_field(3)
    file_sha: str = betterproto.string_field(4)
    is_public: bool = betterproto.bool_field(5)
    platforms_to_sync: List[str] = betterproto.string_field(6)
    request_headers_names: List[str] = betterproto.string_field(7)
    request_headers_values: List[str] = betterproto.string_field(8)


@dataclass
class CCloud_BeginHTTPUpload_Response(betterproto.Message):
    ugcid: float = betterproto.fixed64_field(1)
    timestamp: float = betterproto.fixed32_field(2)
    url_host: str = betterproto.string_field(3)
    url_path: str = betterproto.string_field(4)
    use_https: bool = betterproto.bool_field(5)
    request_headers: List["CCloud_BeginHTTPUpload_ResponseHTTPHeaders"] = betterproto.message_field(6)


@dataclass
class CCloud_BeginHTTPUpload_ResponseHTTPHeaders(betterproto.Message):
    name: str = betterproto.string_field(1)
    value: str = betterproto.string_field(2)


@dataclass
class CCloud_CommitHTTPUpload_Request(betterproto.Message):
    transfer_succeeded: bool = betterproto.bool_field(1)
    appid: int = betterproto.uint32_field(2)
    file_sha: str = betterproto.string_field(3)
    filename: str = betterproto.string_field(4)


@dataclass
class CCloud_CommitHTTPUpload_Response(betterproto.Message):
    file_committed: bool = betterproto.bool_field(1)


@dataclass
class CCloud_GetFileDetails_Request(betterproto.Message):
    ugcid: int = betterproto.uint64_field(1)
    appid: int = betterproto.uint32_field(2)


@dataclass
class CCloud_UserFile(betterproto.Message):
    appid: int = betterproto.uint32_field(1)
    ugcid: int = betterproto.uint64_field(2)
    filename: str = betterproto.string_field(3)
    timestamp: int = betterproto.uint64_field(4)
    file_size: int = betterproto.uint32_field(5)
    url: str = betterproto.string_field(6)
    steamid_creator: float = betterproto.fixed64_field(7)
    flags: int = betterproto.uint32_field(8)
    platforms_to_sync: List[str] = betterproto.string_field(9)
    file_sha: str = betterproto.string_field(10)


@dataclass
class CCloud_GetFileDetails_Response(betterproto.Message):
    details: "CCloud_UserFile" = betterproto.message_field(1)


@dataclass
class CCloud_EnumerateUserFiles_Request(betterproto.Message):
    appid: int = betterproto.uint32_field(1)
    extended_details: bool = betterproto.bool_field(2)
    count: int = betterproto.uint32_field(3)
    start_index: int = betterproto.uint32_field(4)


@dataclass
class CCloud_EnumerateUserFiles_Response(betterproto.Message):
    files: List["CCloud_UserFile"] = betterproto.message_field(1)
    total_files: int = betterproto.uint32_field(2)


@dataclass
class CCloud_Delete_Request(betterproto.Message):
    filename: str = betterproto.string_field(1)
    appid: int = betterproto.uint32_field(2)


@dataclass
class CCloud_Delete_Response(betterproto.Message):
    pass


@dataclass
class CCloud_GetClientEncryptionKey_Request(betterproto.Message):
    pass


@dataclass
class CCloud_GetClientEncryptionKey_Response(betterproto.Message):
    key: bytes = betterproto.bytes_field(1)
    crc: int = betterproto.int32_field(2)


@dataclass
class CCloud_CDNReport_Notification(betterproto.Message):
    steamid: float = betterproto.fixed64_field(1)
    url: str = betterproto.string_field(2)
    success: bool = betterproto.bool_field(3)
    http_status_code: int = betterproto.uint32_field(4)
    expected_bytes: int = betterproto.uint64_field(5)
    received_bytes: int = betterproto.uint64_field(6)
    duration: int = betterproto.uint32_field(7)


@dataclass
class CCloud_ExternalStorageTransferReport_Notification(betterproto.Message):
    host: str = betterproto.string_field(1)
    path: str = betterproto.string_field(2)
    is_upload: bool = betterproto.bool_field(3)
    success: bool = betterproto.bool_field(4)
    http_status_code: int = betterproto.uint32_field(5)
    bytes_expected: int = betterproto.uint64_field(6)
    bytes_actual: int = betterproto.uint64_field(7)
    duration_ms: int = betterproto.uint32_field(8)
    cellid: int = betterproto.uint32_field(9)
    proxied: bool = betterproto.bool_field(10)
    ipv6_local: bool = betterproto.bool_field(11)
    ipv6_remote: bool = betterproto.bool_field(12)


@dataclass
class CCloud_ClientBeginFileUpload_Request(betterproto.Message):
    appid: int = betterproto.uint32_field(1)
    file_size: int = betterproto.uint32_field(2)
    raw_file_size: int = betterproto.uint32_field(3)
    file_sha: bytes = betterproto.bytes_field(4)
    time_stamp: int = betterproto.uint64_field(5)
    filename: str = betterproto.string_field(6)
    platforms_to_sync: int = betterproto.uint32_field(7)
    cell_id: int = betterproto.uint32_field(9)
    can_encrypt: bool = betterproto.bool_field(10)
    is_shared_file: bool = betterproto.bool_field(11)
    realm: int = betterproto.uint32_field(12)


@dataclass
class ClientCloudFileUploadBlockDetails(betterproto.Message):
    url_host: str = betterproto.string_field(1)
    url_path: str = betterproto.string_field(2)
    use_https: bool = betterproto.bool_field(3)
    http_method: int = betterproto.int32_field(4)
    request_headers: List["ClientCloudFileUploadBlockDetailsHTTPHeaders"] = betterproto.message_field(5)
    block_offset: int = betterproto.uint64_field(6)
    block_length: int = betterproto.uint32_field(7)
    explicit_body_data: bytes = betterproto.bytes_field(8)
    may_parallelize: bool = betterproto.bool_field(9)


@dataclass
class ClientCloudFileUploadBlockDetailsHTTPHeaders(betterproto.Message):
    name: str = betterproto.string_field(1)
    value: str = betterproto.string_field(2)


@dataclass
class CCloud_ClientBeginFileUpload_Response(betterproto.Message):
    encrypt_file: bool = betterproto.bool_field(1)
    block_requests: List["ClientCloudFileUploadBlockDetails"] = betterproto.message_field(2)


@dataclass
class CCloud_ClientCommitFileUpload_Request(betterproto.Message):
    transfer_succeeded: bool = betterproto.bool_field(1)
    appid: int = betterproto.uint32_field(2)
    file_sha: bytes = betterproto.bytes_field(3)
    filename: str = betterproto.string_field(4)


@dataclass
class CCloud_ClientCommitFileUpload_Response(betterproto.Message):
    file_committed: bool = betterproto.bool_field(1)


@dataclass
class CCloud_ClientFileDownload_Request(betterproto.Message):
    appid: int = betterproto.uint32_field(1)
    filename: str = betterproto.string_field(2)
    realm: int = betterproto.uint32_field(3)


@dataclass
class CCloud_ClientFileDownload_Response(betterproto.Message):
    appid: int = betterproto.uint32_field(1)
    file_size: int = betterproto.uint32_field(2)
    raw_file_size: int = betterproto.uint32_field(3)
    sha_file: bytes = betterproto.bytes_field(4)
    time_stamp: int = betterproto.uint64_field(5)
    is_explicit_delete: bool = betterproto.bool_field(6)
    url_host: str = betterproto.string_field(7)
    url_path: str = betterproto.string_field(8)
    use_https: bool = betterproto.bool_field(9)
    request_headers: List["CCloud_ClientFileDownload_ResponseHTTPHeaders"] = betterproto.message_field(10)
    encrypted: bool = betterproto.bool_field(11)


@dataclass
class CCloud_ClientFileDownload_ResponseHTTPHeaders(betterproto.Message):
    name: str = betterproto.string_field(1)
    value: str = betterproto.string_field(2)


@dataclass
class CCloud_ClientDeleteFile_Request(betterproto.Message):
    appid: int = betterproto.uint32_field(1)
    filename: str = betterproto.string_field(2)
    is_explicit_delete: bool = betterproto.bool_field(3)


@dataclass
class CCloud_ClientDeleteFile_Response(betterproto.Message):
    pass


@dataclass
class CCloud_ClientConflictResolution_Notification(betterproto.Message):
    appid: int = betterproto.uint32_field(1)
    chose_local_files: bool = betterproto.bool_field(2)


@dataclass
class CCloud_EnumerateUserApps_Request(betterproto.Message):
    pass


@dataclass
class CCloud_EnumerateUserApps_Response(betterproto.Message):
    apps: List["CCloud_EnumerateUserApps_ResponseApps"] = betterproto.message_field(1)


@dataclass
class CCloud_EnumerateUserApps_ResponseApps(betterproto.Message):
    appid: int = betterproto.uint32_field(1)
    totalcount: int = betterproto.int32_field(2)
    totalsize: int = betterproto.int64_field(3)
